@@ -6,13 +6,13 @@
 ** use, modify and/or redistribute the software under the terms of the
 ** CeCILL-A license as circulated by CEA, CNRS and INRIA at the following
 ** URL: "http://www.cecill.info".
-** 
+**
 ** As a counterpart to the access to the source code and rights to copy,
 ** modify and redistribute granted by the license, users are provided
 ** only with a limited warranty and the software's author, the holder of
 ** the economic rights, and the successive licensors have only limited
 ** liability.
-** 
+**
 ** In this respect, the user's attention is drawn to the risks associated
 ** with loading, using, modifying and/or developing or reproducing the
 ** software by the user in light of its specific status of free software,
@@ -23,7 +23,7 @@
 ** their requirements in conditions enabling the security of their
 ** systems and/or data to be ensured and, more generally, to use and
 ** operate it in the same conditions as regards security.
-** 
+**
 ** The fact that you are presently reading this means that you have had
 ** knowledge of the CeCILL-A license and that you accept its terms.
 **
@@ -37,7 +37,7 @@
 **        - MARCOUEILLE Jule
 **        - NOISETTE Pascal
 **        - REDONDY Arthur
-**        - VUCHENER Clément 
+**        - VUCHENER Clément
 **
 */
 /*!
@@ -50,23 +50,37 @@
 #include <string>
 #include <iostream>
 #include <fstream>
+#include <sstream>
 #include <list>
 #include <map>
 #include <vector>
 #include <stack>
+#include <cmath>
 /* -- */
 #include <QObject>
+//#include <QtUiTools>/* for the run-time loading .ui file */
+#include <QRadioButton>
+#include <QTextEdit>
+#include <QTextCodec>
+#include <QTime>
+#include <QDir>
+#include <QGLWidget>/* for the OpenGL Widget */
 /* -- */
+#include "interface/Interface.hpp"
 #include "common/common.hpp"
-#include "common/info.hpp"
+#include "common/Info.hpp"
 #include "common/Errors.hpp"
 #include "common/Tools.hpp"
+#include "common/Session.hpp"        
+#include "common/Message.hpp"
 /* -- */
-
-
-//#include "render/render.hpp"
-#include "render/Render_svg.hpp"
+#include "render/Palette.hpp"
+#include "render/Geometry.hpp"
+#include "render/Hook_event.hpp"
+#include "render/Ruler.hpp"
 #include "render/Render.hpp"
+#include "render/Render_template.hpp"
+#include "render/Render_svg.hpp"
 /* -- */
 #include "trace/values/Values.hpp"
 #include "trace/tree/Interval.hpp"
@@ -80,21 +94,14 @@
 #include "trace/DrawTree.hpp"
 #include "trace/DrawTrace.hpp"
 /* -- */
-#include "parser/File.hpp"
 #include "parser/Parser.hpp"
-#include "parser/ParserPaje.hpp"
-#ifdef WITH_OTF
-#include <otf.h>
-#include "parser/ParserOTF.hpp"
-#endif //WITH_OTF
-#include "parser/ParserVite.hpp"
+#include "parser/ParserFactory.hpp"
+#include "parser/ParsingThread.hpp"
 /* -- */
 #include "statistics/Stats_window.hpp"
 /* -- */
-#include "interface/resource.hpp"
 #include "interface/Interface_graphic.hpp"
 #include "core/Core.hpp"
-#include "interface/parsing_thread.hpp"
 /* -- */
 #ifdef WITH_VBO
 #include "render/Render_alternate.hpp"
@@ -118,48 +125,38 @@ using namespace std;
  **********************************/
 
 Core::Core(int argc, char ** argv){
-    
-    
     bool useGUI;/* if window interface can be displayed */
     //  QString current_path;
 
-
-    app = new QApplication(argc, argv);/* create the Qt application */        
- 
-    
-
+    app = new QApplication(argc, argv);/* create the Qt application */
 
     //  glutInit(&argc, argv);/* use for OpenGL text */
 
     /* Qt uses the default system encoding for QString (used when opening a file) */
-    QTextCodec::setCodecForCStrings(QTextCodec::codecForLocale()); 
-
+    QTextCodec::setCodecForCStrings(QTextCodec::codecForLocale());
 
     useGUI = true;
-    
-  
-    //    cerr << "Core debug: " << QDir::currentPath().toStdString().c_str() << " " << argv[0] << endl;
-    
 
+    //    cerr << "Core debug: " << QDir::currentPath().toStdString().c_str() << " " << argv[0] << endl;
 
     /*  if (!useGUI){
         cerr << QObject::tr("Error: Graphical window cannot be displayed.").toStdString() << endl;
         exit(EXIT_FAILURE);
         }*/
-    
+
     _main_window = NULL;
     _render_opengl =  NULL;
     _trace = NULL;
-    
+
     /* Init of the times */
     _time_start = 0;
     _time_end = 0;
 
     _run_env[0] = new QString(QDir::currentPath().toStdString().c_str());
     _run_env[1] = new QString(argv[0]);
-    
+
     Message::set_interface(this);/* define which interface will receive messages */
-    
+
 
     _state = get_state(argc, argv);
 
@@ -172,28 +169,34 @@ Core::Core(int argc, char ** argv){
      * No trace loaded
      */
     _is_trace_loaded = false;
-    
+
     launch_action(_state);
 }
 
 
 Core::~Core(){
-    
+
     delete app;
-    if(_trace != NULL){
+    if(_trace){
         delete _trace;
     }
-    if (_run_env[0]!=NULL)
+    if (_run_env[0])
         delete _run_env[0];
 
-    if (_run_env[1]!=NULL)
+    if (_run_env[1])
         delete _run_env[1];
 
+
+    if (_render_opengl)
+        _render_opengl->release();
+     
     /* Qt desallocates _main_window and _render_opengl automatically */
-    
+
     //free(_run_env[0]);
     Message::kill();
 
+
+
 }
 
 
@@ -208,59 +211,37 @@ Core::~Core(){
  *
  **********************************/
 
-bool Core::draw_trace(const string & filename, const int format){
+bool Core::draw_trace(const string & filename, const int format) {
 
-    Parser *parser;
+    Parser    *parser = NULL;
+    DrawTrace  drawing_ogl;
 
-    DrawTrace drawing_ogl;
-    
 #ifdef WITH_VBO
     Render<Render_alternate> render(_render_opengl);
 #else
-    Render<Render_opengl> render(_render_opengl);
+    Render<Render_opengl>    render(_render_opengl);
 #endif
 
-
-    QTime time_elapsed;
+    QTime         time_elapsed;
     ostringstream buf_txt;
-    int time_buf;
-    time_buf = 0;
+    int           time_buf = 0;
 
+    // Select the Parser in function of the file's extension
+    ParserFactory::create(&parser, filename);
 
-    // Get the parser in function of the extension of the file
-    const unsigned int position_of_dot = filename.find_last_of('.');
-    if (position_of_dot != string::npos) {
-        if(filename.substr(position_of_dot) == ".trace") {
-            parser = new ParserPaje();
-        }
-#ifdef WITH_OTF
-        else if(filename.substr(position_of_dot) == ".otf") {
-            parser = new ParserOTF();
-        }
-#endif //WITH_OTF
-        else if(filename.substr(position_of_dot) == ".ept") {
-            parser = new ParserVite();
-        }
-        else {
-            Error::set(Error::_BAD_FILE_EXTENSION, Error::_WARNING);
-            parser = new ParserPaje();        
-        }
-    }
-    else {
-        Error::set(Error::_BAD_FILE_EXTENSION, Error::_WARNING);
-        parser = new ParserPaje();        
-    }
-
-
-    parser->set_file_to_parse(filename);
-    
+    // Set application in working state
     QApplication::setOverrideCursor(Qt::WaitCursor);
 
-    if (_DRAW_SVG == format)
-        _render_svg = new Render_svg(&_path_to_export);// svg.init(_path_to_export.c_str());
-        
+    if (_DRAW_SVG == format) {
+        _render_svg = new Render_svg(&_path_to_export);
+        // svg.init(_path_to_export.c_str());
+    }
+    else {
+        // We add the filename to the recent files opened list
+        Session::add_recent_file(QFileInfo(filename.c_str()).absoluteFilePath());
+        _main_window->update_recent_files_menu();
+    }
 
-            
     if (NULL == _trace) { /* no trace is loaded, parse the file */
 
         /* Init data */
@@ -270,19 +251,18 @@ bool Core::draw_trace(const string & filename, const int format){
         Info::Render::_x_max_visible = 0;
 
         _trace = new Trace();
-        
+
         // Init of the thread
-        parsing_thread thread;
-        thread.init(parser, _trace, filename);
+        ParsingThread thread(parser, _trace);
 
         int loaded = 0;
         float loaded_f = 0.0f;/* floating value of the loading file state. (between 0 and 1) */
-        
+
         if(_main_window != NULL) { // If we have a window we show a progress bar
             _progress_dialog = new QProgressDialog(QObject::tr("Parsing"), QObject::tr("Cancel"), 0, 100, _main_window);
             _progress_dialog->setWindowTitle(QObject::tr("Loading of ")+QString::fromStdString(filename));
             _progress_dialog->show();
-            
+
             _main_window->setDisabled(true);
             _progress_dialog->setDisabled(false); // to be able to cancel while parsing
         }
@@ -290,21 +270,21 @@ bool Core::draw_trace(const string & filename, const int format){
         thread.start();
         time_elapsed.start();
         buf_txt.str("");
-        
+
         while(!(parser->is_end_of_parsing() && thread.isFinished())) {
-            loaded_f = parser->get_size_loaded();/* value between 0 and 1. Should be multiplied by 100 to obtain a percentage */
-            loaded = (int)(loaded_f*100.0f);
+            loaded_f = parser->get_percent_loaded();
+            loaded   = (int)(loaded_f*100.0f);
 
 #ifdef WIN32
-	  Sleep(1000);
+	    Sleep(1000);
 #else
-	  sleep(1); // We wait 1 second
+	    sleep(1); // We wait 1 second
 #endif
-          cout << QObject::tr("Loading of the trace : ").toStdString() << loaded << "%" ;
+	    cout << QObject::tr("Loading of the trace : ").toStdString() << loaded << "%" ;
 
 
-          if (loaded_f>0.0f){
-                /* divided by to have in second 1000 since time_elapsed.elapsed() returns ms */
+	    if (loaded_f>0.0f){
+	      // divided by to have in second 1000 since time_elapsed.elapsed() returns ms */
               time_buf = (int)(time_elapsed.elapsed() * (1.0 / loaded_f - 1.0) / 1000);
 
                 if (time_buf>=3600){/* convert second in hour and min */
@@ -316,8 +296,8 @@ bool Core::draw_trace(const string & filename, const int format){
                 else{
                     buf_txt << "Parsing... Remaining: " << time_buf << " s";
                 }
-               
-                cout << "  ~  " << buf_txt.str(); 
+
+                cout << "  ~  " << buf_txt.str();
 
                 if (  (NULL!=_main_window)  &&  (NULL!=_progress_dialog)  )
                     _progress_dialog->setLabelText(QString( buf_txt.str().c_str()));
@@ -331,20 +311,20 @@ bool Core::draw_trace(const string & filename, const int format){
           if(_main_window != NULL) { // If we have a window we show a progress bar
                 update_progress_bar(loaded);
                 QApplication::processEvents();
-            
+
                 if(_progress_dialog->wasCanceled()) {
                     cout << QObject::tr("Canceled at ").toStdString() << loaded << "%" << endl;
                     parser->set_canceled();
                     *Message::get_instance() << QObject::tr("The trace opening was canceled by the user at ").toStdString() << loaded << "%" << Message::ende;
-                   
+
                     break; // Quit the loop
                 }
             }
         }
-        
+
         Error::print_numbers();
         Error::flush("log.txt");
-        
+
         if(_main_window != NULL) { // If we have a window we show a progress bar
             delete _progress_dialog;
             _main_window->setDisabled (false);
@@ -352,8 +332,8 @@ bool Core::draw_trace(const string & filename, const int format){
         // Wait for the end of the thread
         while(!thread.wait()) {
         }
-                
-        delete parser;       
+
+        delete parser;
     }
     else if ( _file_opened != filename) {/* just check if execution is normal */
         *Message::get_instance() << "Try to use file: " << filename << " instead of a previous parsed file: " << _file_opened << Message::ende;
@@ -362,9 +342,16 @@ bool Core::draw_trace(const string & filename, const int format){
     if (Info::Render::_x_min_visible == Info::Render::_x_max_visible){// first time
         _trace->set_interval_constrained(new Interval(0,_trace->get_max_date()));
     }else{
-        _trace->set_interval_constrained(new Interval(Info::Render::_x_min_visible, Info::Render::_x_max_visible));
+        //        _trace->set_interval_constrained(new Interval(Info::Render::_x_min_visible, Info::Render::_x_max_visible));
+        //
+        // WARNING I HAVE COMMENTED THIS LINE BECAUSE I FOUND IT USELESS. I TRIED AFTER AND NO BUG APPEARED
+        // MOREOVER IT ENABLES THE PLUGIN TIME SELECTION
+        //
     }
-         
+//     if(_state & _STATE_IN_AN_INTERVAL) {
+//         _trace->set_interval_constrained(new Interval(_time_start,_time_end));
+//     }
+
 
     if (_DRAW_OPENGL == format) {
         drawing_ogl.build(&render, _trace);
@@ -376,7 +363,7 @@ bool Core::draw_trace(const string & filename, const int format){
     else if (_DRAW_SVG == format) {
 
         Element_pos buf_min, buf_max;
-        
+
         /* Store current view */
         buf_min = Info::Entity::x_min;
         buf_max = Info::Entity::x_max;
@@ -395,49 +382,36 @@ bool Core::draw_trace(const string & filename, const int format){
         if(Info::Entity::x_max >= Info::Render::_x_min_visible) {
             Info::Entity::x_min = Info::Render::_x_min_visible;
         }
+
         if(_time_end > _time_start && _time_end <= Info::Entity::x_max) {
             Info::Entity::x_max = _time_end;
         }
-//         /* Change current view to fit the user zoom */
-//         if(_state == _STATE_EXPORT_FILE_IN_INTERVAL) { // Command line
-//             if(Info::Entity::x_max >= _time_start) {
-//                 Info::Entity::x_min = _time_start;
-//             }
-//             if(_time_end > _time_start && _time_end <= Info::Entity::x_max) {
-//                 Info::Entity::x_max = _time_end;
-//             }
-//         }
-//         else { // Export launched by the graphical interface
-//             Info::Entity::x_min = Info::Render::_x_min_visible;
-//             Info::Entity::x_max = Info::Render::_x_max_visible;
-//         }
-
+        
         Render<Render_svg> render(_render_svg);
         drawing_ogl.build(&render, _trace);
-  
+
         delete _render_svg;/* Release the svg render */
 
         /* Restore previous view */
         Info::Entity::x_min = buf_min;
         Info::Entity::x_max = buf_max;
-        
+
         // svg.end();
     }else{/* error */
         *Message::get_instance() << "No kind of render recognized" << Message::ende;
         return false;
     }
     //   std::cout << _trace->get_max_date().get_value() << __FILE__ << __LINE__ << std::endl;
-       
-    
+
+
     _is_trace_loaded = true;
-    QApplication::restoreOverrideCursor();  
+    QApplication::restoreOverrideCursor();
 
     return true;
 }
 
 
 int Core::run(){
-
     /*
      * If a window is displayed, enter in the Qt event loop.
      */
@@ -463,7 +437,7 @@ int Core::run(){
  **********************************/
 
 int Core::get_state(int argc, char** argv){
- 
+
 
     /*The following statics must be initialised in case their are not overrided by a command line request*/
     /*  Svg::set_height_factor (20);
@@ -477,7 +451,7 @@ int Core::get_state(int argc, char** argv){
         return _STATE_LAUNCH_GRAPHICAL_INTERFACE;
     }
 
-    int state = _STATE_DISPLAY_HELP; 
+    int state = _STATE_DISPLAY_HELP;
 
 
     for(int i = 1 ; i < argc ; i ++) {
@@ -588,9 +562,9 @@ int Core::get_state(int argc, char** argv){
         //             //  Svg::set_scale_frequency(scale);
         //         }
         //     }
-        //     else 
+        //     else
              if((string)argv[i] == "-interval") {
-              
+
                 if (!(i +2 < argc)) {
                     cerr << "interval argc" << endl;
                     return _STATE_UNKNOWN;
@@ -603,10 +577,10 @@ int Core::get_state(int argc, char** argv){
                     cerr << "interval -" << endl;
                     return _STATE_UNKNOWN;
                 }
-	   
+
                 double t1 = atof(argv[i+1]);
                 double t2 = atof(argv[i+2]);
-	   
+
                 if (t1 >= t2) {
                     cerr << "interval neg" << endl;
                     return _STATE_UNKNOWN;
@@ -619,7 +593,7 @@ int Core::get_state(int argc, char** argv){
                 cerr << "the argument "<< argv[i] << " is unknown." << endl;
                 return _STATE_DISPLAY_HELP;
             }
-	  
+
         }
         else{ // This is a file to open
             _file_opened = argv[i];
@@ -639,7 +613,7 @@ void Core::extract_times(const char *name) {
     if(name[strlen(name)-1] == ']') {
         has_time_end = true;
     }
-            
+
     if(has_time_start && has_time_end) {
         string start = temp.substr(temp.find('[')+1, temp.find(']')-1);
         temp = temp.substr(temp.find(']')+1);
@@ -662,21 +636,20 @@ void Core::extract_times(const char *name) {
 void Core::launch_action(int state, void* arg) {
 
     DrawTrace buf;
-    QGLFormat format(QGL::HasOverlay);
+    QGLFormat format (QGL::HasOverlay);
 
     switch(state) {
 
     case _STATE_DISPLAY_HELP :
-        display_help();
+        display_help ();
         break;
-        
-    case _STATE_LAUNCH_GRAPHICAL_INTERFACE :
+
     case _STATE_OPEN_FILE:
+        message << QObject::tr("Opening the file: ").toStdString ()+_file_opened
+                << Message::endi;
 
+    case _STATE_LAUNCH_GRAPHICAL_INTERFACE :
 
-        if ( _STATE_OPEN_FILE == state)
-            message << QObject::tr("Opening the file: ").toStdString ()+_file_opened << Message::endi;
-        
         _main_window = new Interface_graphic (this);/* launch the window interface */
         Message::set_interface (_main_window);/* define which interface will receive messages */
 
@@ -688,23 +661,26 @@ void Core::launch_action(int state, void* arg) {
 #endif
 
         if ( NULL == _render_opengl){
-            message <<  QObject::tr("Cannot allocate memory for an OpengGL instance").toStdString () << Message::ende;
+            message << QObject::tr("Cannot allocate memory for an OpengGL instance").toStdString ()
+                    << Message::ende;
             break;
         }
 
         if (NULL == _render_opengl->context()){
-            message <<  QObject::tr("Cannot allocate an OpengGL context").toStdString () << Message::ende;
+            message << QObject::tr("Cannot allocate an OpengGL context").toStdString ()
+                    << Message::ende;
             break;
         }
-        
+
         if (false == _render_opengl->isValid()){
-            message <<  QObject::tr("Invalid context: OpenGL is not supported on your system").toStdString () << Message::ende;
+            message << QObject::tr("Invalid context: OpenGL is not supported on your system").toStdString ()
+                    << Message::ende;
          }
 
 
         // if (false == _render_opengl->format().hasOverlay ()){
         //     /* If no overlay was created, check the original format */
- 
+
         //     if (false == _render_opengl->context()->requestedFormat().hasOverlay())
         //         message <<  QObject::tr("No overlay format originally created").toStdString () << Message::endw;
         //     else
@@ -713,20 +689,18 @@ void Core::launch_action(int state, void* arg) {
 
         _main_window->bind_render_area((QGLWidget*)_render_opengl);
 
-        if ( _STATE_OPEN_FILE == state){
+        if ( _STATE_OPEN_FILE == state ){
             _main_window->opening_file(_file_opened);/* Must be called after binding the render area to the main window */
-            
-            if(false==draw_trace(_file_opened, _DRAW_OPENGL))
-                message <<  QObject::tr("Draw trace failed").toStdString () << Message::ende;
+            if( false == draw_trace(_file_opened, _DRAW_OPENGL) )
+                message << QObject::tr("Draw trace failed").toStdString ()
+                        << Message::ende;
         }
 
         _is_window_displayed = true;
         break;
 
     case _STATE_OPEN_FILE_IN_AN_INTERVAL:
-        
-        launch_action(_STATE_OPEN_FILE, NULL); 
-        
+        launch_action(_STATE_OPEN_FILE, NULL);
         break;
 
     case _STATE_RELEASE_RENDER_AREA:
@@ -734,10 +708,11 @@ void Core::launch_action(int state, void* arg) {
             message << "Close file : an error occured with trace releasing." << Message::ende;
         _file_opened.clear();
 
+
         if (false == _is_trace_loaded){
             *Message::get_instance() << "Try to release a render area whereas no file was loaded" << Message::ende;
         }else{
-             _is_trace_loaded = false;
+            _is_trace_loaded = false;
         }
 
         if (NULL == _trace){
@@ -747,7 +722,6 @@ void Core::launch_action(int state, void* arg) {
             _trace = NULL;
         }
 
-       
         _render_opengl->updateGL();
 
         /* Release all data */
@@ -755,7 +729,6 @@ void Core::launch_action(int state, void* arg) {
 
         break;
 
-        
     case _STATE_EXPORT_FILE:
         if (_file_opened.empty()){
             *Message::get_instance() << "Please to previously open a trace." << Message::endw;
@@ -763,9 +736,9 @@ void Core::launch_action(int state, void* arg) {
         }
 
         *Message::get_instance() << "export of " << _file_opened << " to " << _path_to_export << Message::endi;
-        
+
         draw_trace(_file_opened, _DRAW_SVG);
-       
+
         break;
 
     case _STATE_EXPORT_FILE_IN_INTERVAL:
@@ -782,16 +755,16 @@ void Core::launch_action(int state, void* arg) {
         else{
             cout << "the end of the trace.";
         }
-        
+
         // Svg::set_interval(_time_start, _time_end);
         draw_trace(_file_opened, _DRAW_SVG);
-        
+
         break;
 
     case _STATE_RENDER_AREA_CHANGE_TRANSLATE:
         _render_opengl->change_translate( *((int*)arg) );
         break;
-	
+
     case _STATE_RENDER_AREA_CHANGE_SCALE:
         _render_opengl->change_scale( *((Element_pos*)arg) );
         break;
@@ -820,22 +793,22 @@ void Core::launch_action(int state, void* arg) {
     case _STATE_RENDER_AREA_REGISTERED_TRANSLATE:
         _render_opengl->registered_translate( *((int*)arg) );
         break;
-	
+
     case _STATE_AJUST_SCROLL_BARS:
-	//	cerr << "console: x_max " << ((Element_pos*)arg)[0] << " | y_max " << ((Element_pos*)arg)[1] << endl;
-	_main_window->set_scroll_bars_length( ((Element_pos*)arg)[0], ((Element_pos*)arg)[1] );
-	break;
-	
+        //        cerr << "console: x_max " << ((Element_pos*)arg)[0] << " | y_max " << ((Element_pos*)arg)[1] << endl;
+        _main_window->set_scroll_bars_length( ((Element_pos*)arg)[0], ((Element_pos*)arg)[1] );
+        break;
+
     case _STATE_REFRESH_SCROLL_BARS:
-	_main_window->linking_scroll_bars( ((Element_pos*)arg)[0], ((Element_pos*)arg)[1] );
-	break;
-	
+        _main_window->linking_scroll_bars( ((Element_pos*)arg)[0], ((Element_pos*)arg)[1] );
+        break;
+
     case _STATE_ZOOM_BOX_VALUE:
         _main_window->change_zoom_box_value( *((int*)arg) );
         break;
 
     case _STATE_RENDER_DISPLAY_INFORMATION:
-        
+
         if(_trace != NULL){
             buf.display_information(_trace, Info::Render::_info_x, Info::Render::_info_y, Info::Render::_info_accurate);
         }
@@ -845,24 +818,28 @@ void Core::launch_action(int state, void* arg) {
         _render_opengl->updateGL();
         break;
 
+    case _STATE_RENDER_SHOW_MINIMAP:
+        _render_opengl->show_minimap();
+        break;
+
     default:/* like _STATE_UNKNOWN */
         display_help();
         warning(string("Cannot determine the arguments past. Please check the correct syntax."));
-       
+
     }
 }
 
 
 void Core::display_help(){
-    cout << endl 
+    cout << endl
          << "Usage: vite [OPTION...] [[-f] inputfile] [-e outputfile] " << endl
          << endl
          << "  -h                 display this help                  " << endl
          << "  -f inputfile       open the inputfile                 " << endl
          << "  -a                 display all the inputfile (Default)" << endl
-         << "  -t [T0]:[T1]       display the interval [T0:T1] (Default: T0 = 0 and T1 = Tmax)" << endl 
+         << "  -t [T0]:[T1]       display the interval [T0:T1] (Default: T0 = 0 and T1 = Tmax)" << endl
          << "  -e outputfile      export the trace file in the svg format to outpufile"        << endl
-	 << endl;
+         << endl;
 }
 
 
@@ -877,7 +854,7 @@ void Core::display_help(){
  *
  *
  **********************************/
- 
+
 void Core::error(const string &s) const{
     cerr << "ERROR: " << s <<endl;
 }
@@ -931,7 +908,7 @@ void Core::export_variable(Variable *var, string filename) {
         double second_value = 0.;
         const double min = var->get_min().get_value();
         const double max = var->get_max().get_value();
-        
+
         for(list<pair<Date, Double> >::const_iterator value = variable_values->begin();
             value != variable_values->end();
             value++) {
@@ -939,10 +916,10 @@ void Core::export_variable(Variable *var, string filename) {
             second_value =((*value).second.get_value()-min)/(max-min) ;
             file << first_value << "\t" << second_value << endl;
         }
-    
+
         file.close();
     }
-    else { 
+    else {
         // Error
         _main_window->error("Unable to open " + filename + " in order to export the counter");
     }
