@@ -46,14 +46,27 @@
  */
 #include <fstream>
 #include <string>
+#include <iostream>
+#include <sstream>
 #include <stack>
 #include <map>
 #include <list>
 /* -- */
 #include <QObject>
+#include <QGLWidget>/* for the OpenGL Widget */
+#include <QtUiTools>/* for the run-time loading .ui file */
+#include <QCloseEvent>
+#include <QDir>
+#include <QtGui/QFileDialog>
+#include <QtGui/QRadioButton>
+#include <QtGui/QTextEdit>
+#include <QtGui/QCheckBox>
+#include <QtGui/QMessageBox>
+/* -- */
+#include "interface/resource.hpp"
 /* -- */
 #include "common/common.hpp"
-#include "common/info.hpp"
+#include "common/Info.hpp"
 #include "common/Tools.hpp"
 #include "common/Session.hpp"
 /* -- */
@@ -62,19 +75,13 @@
 #include "trace/Entitys.hpp"
 #include "trace/Trace.hpp"
 /* -- */
-//#include "render/render.hpp"
-//#include "render/render_svg.hpp"
-/* -- */
-#include "statistics/Stats_window.hpp"
-/* -- */
-#include "interface/resource.hpp"
-//#include "interface/help.hpp"
-//#include "interface/interface.hpp"
+#include "interface/Interface.hpp"
 #include "core/Core.hpp"
+#include "interface/Settings_window.hpp"
 #include "interface/Interface_graphic.hpp"
-//#include "resource.hpp"
-
+/* -- */
 #include "plugin/Command_window.hpp"
+#include "plugin/Plugin_window.hpp"
 
 
 using namespace std;
@@ -107,7 +114,8 @@ Interface_graphic::Interface_graphic(Core* core, QWidget *parent):QMainWindow(pa
 
     setMouseTracking (true);/* to catch mouse events */
     
-    _stats_window = NULL; // Creation when needed (when it has to be shown)
+    _plugin_window = NULL;
+    _ui_settings = NULL;
     //    _cmd_window = NULL;
     _cmd_window = new Command_window(this,this);
 
@@ -117,9 +125,14 @@ Interface_graphic::Interface_graphic(Core* core, QWidget *parent):QMainWindow(pa
 
 Interface_graphic::~Interface_graphic(){
 
-    if(_stats_window != NULL)
-        delete _stats_window;
- 
+    if(_plugin_window != NULL)
+        delete _plugin_window;
+
+    if (_cmd_window != NULL){
+        delete _cmd_window;
+        _cmd_window = NULL;
+    }
+        
     /* Qt desallocates this, _ui_info_window and _render_area automatically */
 }
 
@@ -187,7 +200,6 @@ void Interface_graphic::load_windows(){
     CKFP(_ui_fullscreen_menu = qFindChild<QAction*>(this, "fullscreen"), "Cannot find the fullscreen menu in the .ui file");
     CKFP(_ui_info_trace_text = qFindChild<QTextEdit*>(_ui_info_window, "info_trace_text"), "Cannot find the info_trace_text QTextEdit widget in the .ui file");
     CKFP(_ui_info_selection_text = qFindChild<QTextEdit*>(_ui_info_window, "info_selection_text"), "Cannot find the info_selection_text QTextEdit widget in the .ui file");
-    CKFP(_ui_toolbar_check = qFindChild<QAction*>(this, "toolbar_menu"), "Cannot find the tool bar checker in the .ui file");   
     CKFP(_ui_toolbar = qFindChild<QToolBar*>(this, "toolBar"), "Cannot find the tool bar in the .ui file");   
 
     CKFP(_ui_x_scroll = qFindChild<QScrollBar*>(this, "x_scroll"), "Cannot find the horizontal scroll bar in the .ui file");
@@ -195,24 +207,11 @@ void Interface_graphic::load_windows(){
 
     CKFP(_ui_zoom_box = qFindChild<QComboBox*>(this, "zoom_box"), "Cannot find the zoom box in the .ui file");   
 
+    CKFP(_ui_recent_files_menu = qFindChild<QMenu*>(this, "menuRecent_Files"), "Cannot find the button \"menuRecent_Files\" in the .ui file");   
+
     //  CKFP(Info::Render::_ui_render_min_value = qFindChild<QLabel*>(this, "render_min_value"), "Cannot find the render_min_value label in the .ui file");
     // CKFP(Info::Render::_ui_render_max_value = qFindChild<QLabel*>(this, "render_max_value"), "Cannot find the render_max_value label in the .ui file");
 
-    /* Export window */
-    CKFP(_ui_export_ok_button = qFindChild<QPushButton*>(_ui_time_selection_export, "option_export_ok"), "Cannot find the ok push button in the export dialog .ui file");   
-    CKFP(_ui_min_time_export = qFindChild<QLineEdit*>(_ui_time_selection_export, "min_time"), "Cannot find the ok push button in the export dialog .ui file");
-
-    CKFP(_ui_max_time_export = qFindChild<QLineEdit*>(_ui_time_selection_export, "max_time"), "Cannot find the ok push button in the export dialog .ui file");
-
-    CKFP(_ui_CheckBox_time_export = qFindChild<QCheckBox*>(_ui_time_selection_export, "export_all"), "Cannot find the Line text \"export_all\" in the export dialog .ui file");   
-
-    CKFP(_ui_wide_export = qFindChild<QLineEdit*>(_ui_time_selection_export, "wide"), "Cannot find the Line text \"wide\" in the export dialog .ui file");   
-    CKFP(_ui_grow_export = qFindChild<QLineEdit*>(_ui_time_selection_export, "grow"), "Cannot find the Line text \"grow\" in the export dialog .ui file");   
-    CKFP(_ui_accuracy_export = qFindChild<QLineEdit*>(_ui_time_selection_export, "accuracy"), "Cannot find the Line text \"accuracy\" in the export dialog .ui file");   
-    CKFP(_ui_CheckBox_wide_export = qFindChild<QCheckBox*>(_ui_time_selection_export, "checkBox_wide"), "Cannot find the Line text \"wide\" in the export dialog .ui file");   
-    CKFP(_ui_CheckBox_grow_export = qFindChild<QCheckBox*>(_ui_time_selection_export, "checkBox_grow"), "Cannot find the Line text \"grow\" in the export dialog .ui file");   
-    CKFP(_ui_CheckBox_accuracy_export = qFindChild<QCheckBox*>(_ui_time_selection_export, "checkBox_accuracy"), "Cannot find the Line text \"accuracy\" in the export dialog .ui file");   
-    CKFP(_ui_more_options_export = qFindChild<QCommandLinkButton*>(_ui_time_selection_export, "more_options"), "Cannot find the button \"more_options\" in the export dialog .ui file");   
 
     /* Help window */
     CKFP(_ui_help_ok_button = qFindChild<QPushButton*>(_ui_help_window, "help_ok"), "Cannot find the ok push button in the help dialog .ui file");     
@@ -224,12 +223,17 @@ void Interface_graphic::load_windows(){
     /* connect(_ui_kind_of_export_choice, SIGNAL(accepted()),
             this, SLOT(choice_of_the_export_pressed()));
     */
-    connect(_ui_export_ok_button, SIGNAL(pressed()),
-            this, SLOT(option_export_ok_pressed()));
-
+    
     connect(_ui_help_ok_button, SIGNAL(clicked()),
             _ui_help_window, SLOT(close()));
 
+    
+    // For the recent files menu
+    for (int i = 0 ; i < Session::_MAX_NB_RECENT_FILES ; ++ i) {
+        _recent_file_actions[i] = NULL;
+    }
+    update_recent_files_menu();
+    
     /* 
        Special function of Qt which allows methods declared as slots and which name are 'on_[widget]_[action]()' to be called when the 'widget' triggered the signal corresponding to 'action'.
        /!\ -> use NULL as argument, else messages will be duplicated! 
@@ -577,8 +581,9 @@ void Interface_graphic::on_export_file_triggered(){
         _ui_counter_choice_to_export->show();
     }
     else if (extension=="png" || extension=="jpg" || extension=="jpeg"){
-        //    else if (_ui_png_export_button->isChecked()) {
-        QImage tempImage = _ui_render_area->grabFrameBuffer(true);/* true = with alpha channel */ // new QImage( pPixelData, width, height, QImage::Format_RGB32);
+
+        QImage tempImage = _ui_render_area->grabFrameBuffer(true);/* true = with alpha channel */ 
+        
         
         if ( !tempImage.save( filename, extension.toUpper().toStdString().c_str()) ){
             error("The trace cannot be exported into "+filename.toStdString()+" file.");
@@ -592,6 +597,29 @@ void Interface_graphic::on_export_file_triggered(){
     }
 }
 
+void Interface_graphic::on_reload_triggered() {
+    if(_is_rendering_trace) {
+        _core->launch_action(Core::_STATE_RELEASE_RENDER_AREA);
+        _core->draw_trace(_trace_path, Core::_DRAW_OPENGL);
+    }
+}
+
+void Interface_graphic::open_recent_file() {
+    QAction *action = qobject_cast<QAction *>(sender());
+    if (action)
+        open_trace(action->data().toString());
+}
+
+void Interface_graphic::on_clear_recent_files_triggered() {
+    Session::clear_recent_files();
+    // We remove the elements from the menu
+    for(int i = 0 ; i < Session::_MAX_NB_RECENT_FILES ; ++ i) {
+        if(_recent_file_actions[i] != NULL) {
+            _ui_recent_files_menu->removeAction(_recent_file_actions[i]);
+        }
+    }
+}
+
 void Interface_graphic::counter_choosed_triggered() {
     // Il faut faire choisir le nom du fichier! et enfin on peut lancer l'action !
   // const QString path_by_default = QString(_trace_path.substr(0, _trace_path.find_last_of('.')).c_str()) + ".txt";
@@ -663,11 +691,20 @@ void Interface_graphic::on_close_triggered(){
     _ui_info_selection_text->clear();/* Clear the current text (if exists) */
     _ui_info_window->hide();/* Hide the informative window */
 
-    //_stats_window->close_window();
+    if(_ui_settings) {
+        _ui_settings->hide();
+    }
 
-    _core->launch_action(Core::_STATE_RELEASE_RENDER_AREA);
+    if(_plugin_window != NULL) {
+        _plugin_window->hide();
+        //_plugin_window->clear_plugins();
+    }
 
-   
+   _core->launch_action(Core::_STATE_RELEASE_RENDER_AREA);
+    if (_cmd_window){
+        delete _cmd_window;
+        _cmd_window = NULL;
+    }
     
     _is_rendering_trace = false; 
 
@@ -678,7 +715,6 @@ void Interface_graphic::on_close_triggered(){
    
 
     this->setWindowTitle(QString("ViTE"));
-
 }
 
 
@@ -706,16 +742,22 @@ void Interface_graphic::on_fullscreen_triggered(){
 }
 
 void Interface_graphic::on_toolbar_menu_triggered(){
-    if (_ui_toolbar_check->isChecked()){
-        _ui_toolbar_check->setText("Hide toolbar");
+    static bool checked = true;/* Suppose that toolbar_menu is initially checked */
+
+    checked = !checked;
+  
+    if (!checked)
         _ui_toolbar->show();
-    }
-    else{
-        _ui_toolbar_check->setText("Show toolbar");
+    else
         _ui_toolbar->hide();
-    }
 }
 
+
+void Interface_graphic::on_minimap_menu_triggered(){
+    _core->launch_action(Core::_STATE_RENDER_SHOW_MINIMAP,NULL);
+}
+
+
 void Interface_graphic::on_show_info_triggered(){
     if (_ui_info_window->isHidden())
         _ui_info_window->show();
@@ -753,27 +795,41 @@ void Interface_graphic::on_about_triggered(){
                           "</a>.<br /><br />"));
 }
 
-void Interface_graphic::on_show_stats_triggered(){
+void Interface_graphic::on_show_plugins_triggered() {
+
+
+
+
     if(_core->get_trace() != NULL) {
 
-        if(_stats_window == NULL) { //Creation of the window
-            _stats_window = new Stats_window(this);
+        if(_plugin_window == NULL) { //Creation of the window
+            _plugin_window = new Plugin_window(_core, this);
         }
-
-        _stats_window->set_filename(_core->get_filename());
-        _stats_window->set_trace(_core->get_trace());
-        _stats_window->init_window();
-        _stats_window->show(); 
+        _plugin_window->update_trace();
+        _plugin_window->show();
+        // TODO create actions as for the recent files menu in order to open at the good tab
     }
     else {
-        error("Must load a trace before viewing stats");
+        error("Must load a trace before applying plugins");
     }
 }
 
+
+void Interface_graphic::on_show_settings_triggered() {
+
+    if (!_ui_settings) {
+        _ui_settings = new Settings_window(_core);
+        connect(_ui_settings, SIGNAL(settings_changed()), this, SLOT(update_settings()));
+        // To close the window when we quit the application
+        connect(quit, SIGNAL(triggered()), _ui_settings, SLOT(close()));
+    }
+    _ui_settings->show();
+}
+
 void Interface_graphic::on_actionCommand_triggered(){
-    //TODO
     if(_core->get_trace()){
-        //        _cmd_window = new Command_window(_core->get_trace(),this);
+        if (!_cmd_window)
+            _cmd_window = new Command_window (this, this);
         _cmd_window->set_trace(_core->get_trace());
         _cmd_window->init_window();
         _cmd_window->show();
@@ -894,6 +950,10 @@ void Interface_graphic::on_scale_container_state_valueChanged(int new_value){
 void Interface_graphic::closeEvent(QCloseEvent *event){
 
     //   _ui_help_widget->close();
+    /* Now, release render */
+    if(_is_rendering_trace)
+        _core->launch_action(Core::_STATE_RELEASE_RENDER_AREA);
+    _is_rendering_trace = false;
 
     if(isEnabled()){
         event->accept();/* accept to hide the window for a further destruction */
@@ -901,6 +961,8 @@ void Interface_graphic::closeEvent(QCloseEvent *event){
     else{
         event->ignore();
     }
+    if(_ui_settings)
+        _ui_settings->close();
 }
 
 const std::string Interface_graphic::get_filename() const{
@@ -912,6 +974,28 @@ Core * Interface_graphic::get_console(){
     return _core;
 }
 
+void Interface_graphic::update_recent_files_menu() {
+    const QStringList filenames = Session::get_recent_files();
+    QString absoluteFilename;
+
+    for (int i = 0 ; i < Session::_MAX_NB_RECENT_FILES && i < filenames.size() ; ++ i) {
+        if(_recent_file_actions[i] != NULL) {
+            delete _recent_file_actions[i];
+        }
+        _recent_file_actions[i] = new QAction(this);
+        _recent_file_actions[i]->setVisible(true);
+        absoluteFilename = QFileInfo(filenames[i]).absoluteFilePath();
+
+        const QString text = tr("&%1 %2").arg(i+1).arg(absoluteFilename);
+        _recent_file_actions[i]->setText(text);
+        _recent_file_actions[i]->setData(absoluteFilename);
+        connect(_recent_file_actions[i], SIGNAL(triggered()),
+                this, SLOT(open_recent_file()));
+        
+        // We add to the menu
+        _ui_recent_files_menu->addAction(_recent_file_actions[i]);
+    }
+}
 
 void Interface_graphic::dragEnterEvent(QDragEnterEvent *event) {
     setBackgroundRole(QPalette::Highlight);
@@ -947,6 +1031,8 @@ void Interface_graphic::dropEvent(QDropEvent *event) {
 
 
 void Interface_graphic::open_trace(const QString &filename) {
+
+    // We open the trace
     if (_is_rendering_trace == true){ /* open a new process */
         
         QStringList arguments = (QStringList() << filename);
@@ -972,5 +1058,13 @@ void Interface_graphic::open_trace(const QString &filename) {
         
         if(false==_core->draw_trace(filename.toStdString(), Core::_DRAW_OPENGL))
             error("Draw trace failed");
-    }/* end else of if (_is_rendering_trace == true) */   
+    }/* end else of if (_is_rendering_trace == true) */
+}
+
+void Interface_graphic::update_settings() {
+    // Reload plugins
+    if(_plugin_window) {
+        _plugin_window->reload_plugins();
+    }
+    cout << "Settings changed, need to update classes" << endl;
 }
