@@ -54,6 +54,7 @@
 #include "trace/values/Values.hpp"
 #include "trace/EntityValue.hpp"
 #include "trace/EntityTypes.hpp"
+#include "trace/Entitys.hpp"
 #include "trace/Trace.hpp"
 /* -- */
 #include "parser/ParserDefinitionOTF.hpp"
@@ -65,6 +66,7 @@ uint64_t ParserEventOTF::_cur_time = 0;
 uint64_t ParserEventOTF::_min_time = 0;
 uint64_t ParserEventOTF::_max_time = 0;
 map <const String, Container *, String::less_than> ParserEventOTF::_containers;
+std::map<int, Marker > ParserEventOTF::_marker;
 
 ParserEventOTF::ParserEventOTF() : _handlers(OTF_HandlerArray_open()) {
 }
@@ -75,26 +77,15 @@ ParserEventOTF::~ParserEventOTF() {
 }
 
 void ParserEventOTF::set_handlers(Trace *t) {
-    OTF_HandlerArray_setHandler(_handlers, (OTF_FunctionPointer*) handler_enter, OTF_ENTER_RECORD);
-    OTF_HandlerArray_setFirstHandlerArg(_handlers, t, OTF_ENTER_RECORD);
-
-    OTF_HandlerArray_setHandler(_handlers, (OTF_FunctionPointer*) handler_leave, OTF_LEAVE_RECORD);
-    OTF_HandlerArray_setFirstHandlerArg(_handlers, t, OTF_LEAVE_RECORD);
-
-    OTF_HandlerArray_setHandler(_handlers, (OTF_FunctionPointer*) handler_send_message, OTF_SEND_RECORD);
-    OTF_HandlerArray_setFirstHandlerArg(_handlers, t, OTF_SEND_RECORD);
-
-    OTF_HandlerArray_setHandler(_handlers, (OTF_FunctionPointer*) handler_receive_message, OTF_RECEIVE_RECORD);
-    OTF_HandlerArray_setFirstHandlerArg(_handlers, t, OTF_RECEIVE_RECORD);
-        
-    OTF_HandlerArray_setHandler(_handlers, (OTF_FunctionPointer*) handler_begin_process, OTF_BEGINPROCESS_RECORD);
-    OTF_HandlerArray_setFirstHandlerArg(_handlers, t, OTF_BEGINPROCESS_RECORD);
-
-    OTF_HandlerArray_setHandler(_handlers, (OTF_FunctionPointer*) handler_end_process, OTF_ENDPROCESS_RECORD);
-    OTF_HandlerArray_setFirstHandlerArg(_handlers, t, OTF_ENDPROCESS_RECORD);
-
-    OTF_HandlerArray_setHandler(_handlers, (OTF_FunctionPointer*) handler_counter, OTF_COUNTER_RECORD);
-    OTF_HandlerArray_setFirstHandlerArg(_handlers, t, OTF_COUNTER_RECORD);
+    SET_HANDLER(_handlers, handler_enter, t, OTF_ENTER_RECORD);
+    SET_HANDLER(_handlers, handler_leave, t, OTF_LEAVE_RECORD);
+    SET_HANDLER(_handlers, handler_send_message, t, OTF_SEND_RECORD);
+    SET_HANDLER(_handlers, handler_receive_message, t, OTF_RECEIVE_RECORD);
+    SET_HANDLER(_handlers, handler_begin_process, t, OTF_BEGINPROCESS_RECORD);
+    SET_HANDLER(_handlers, handler_end_process, t, OTF_ENDPROCESS_RECORD);
+    SET_HANDLER(_handlers, handler_counter, t, OTF_COUNTER_RECORD);
+    SET_HANDLER(_handlers, handler_defmarker, t, OTF_DEFMARKER_RECORD);
+    SET_HANDLER(_handlers, handler_marker, t, OTF_MARKER_RECORD);
 }
 
 int ParserEventOTF::handler_begin_process(void *trace, uint64_t time, uint32_t process) {
@@ -110,20 +101,18 @@ int ParserEventOTF::handler_begin_process(void *trace, uint64_t time, uint32_t p
     
     ContainerType *process_container_type = t->search_container_type(String(process_group._name));
     Container *parent_container = t->search_container(String(process_parent._name));
-//     if(_containers.find(container) != _containers.end()) {
-//         temp_container = _containers[container];
-//     }
-//     else {
-//         temp_container = trace.search_container(container);
-//         _containers[container] = temp_container;
-//     }
 
+    if(t->search_container(String(current_process._name))) {
+        return OTF_RETURN_OK; // Already created so we quit the function
+    }
 
     if(process_container_type == 0) {
         Error::set(Error::_UNKNOWN_CONTAINER_TYPE + process_group._name, Error::_ERROR);
     }
     if(parent_container == 0 && process_parent._name != "0") {
-        Error::set(Error::_UNKNOWN_CONTAINER + process_parent._name, Error::_ERROR);
+        // Create the parent process
+        handler_begin_process(trace, time, current_process._parent);
+        parent_container = t->search_container(String(process_parent._name));
     }
 
     t->create_container(d, n, process_container_type, parent_container, extra_fields);
@@ -203,7 +192,6 @@ int ParserEventOTF::handler_enter (void *trace, uint64_t time, uint32_t func_id,
 
     if(!temp_function._is_defined) {
         temp_function._is_defined = true;
-        Color *c = temp_function._color;
         
         Name name_temp(temp_function._name, "");
         map<string, Value *> opt;
@@ -214,16 +202,13 @@ int ParserEventOTF::handler_enter (void *trace, uint64_t time, uint32_t func_id,
         
         if(temp_function._file_source != 0) {
             source_file_locator = ParserDefinitionOTF::get_fileline_by_id(temp_function._file_source);
+            source_file = ParserDefinitionOTF::get_filesource_by_id(source_file_locator._file_id);
         }
 
-        source_file = ParserDefinitionOTF::get_filesource_by_id(source_file_locator._file_id);
-        
         if(source_file._name != "") {
             opt["File name"] = new String(source_file._name);
             opt["Line"] = new Integer(source_file_locator._line_number);
         }
-        //We add the color
-        opt["Color"] = c;
         t->define_entity_value(name_temp, t->search_entity_type(function_group_name), opt);
     }
 
@@ -322,7 +307,67 @@ int ParserEventOTF::handler_counter(void *trace, uint64_t time, uint32_t proc_id
 
     t->set_variable(d, temp_variable_type, temp_container, value, extra_fields);
     return OTF_RETURN_OK;
+}
 
+int ParserEventOTF::handler_defmarker(void *, uint32_t stream, uint32_t id, const char *name, uint32_t type) {
+    Marker temp = {name, stream, type};
+    ParserEventOTF::_marker[id] = temp;
+    return OTF_RETURN_OK;
+}
+
+int ParserEventOTF::handler_marker(void *trace, uint64_t time, uint32_t proc_id, uint32_t id, const char *text) {
+    Trace *t = (Trace *)trace;
+    Date d = (double)time/(double)ParserDefinitionOTF::get_ticks_per_second();
+
+    Marker temp_marker = _marker[id];
+    Process temp_proc = ParserDefinitionOTF::get_process_by_id(proc_id);
+    Container *temp_container = NULL;
+    map<string, Value *> extra_fields;
+
+    String str_eventType(temp_marker._name);
+    String str_event(text);
+    Name n(text, "");
+    EventType *event_type = t->search_event_type(str_eventType);
+
+    ProcessGroup process_group = ParserDefinitionOTF::get_processgroup_by_process(proc_id);
+    ContainerType *temp_container_type = t->search_container_type(String(process_group._name));
+    EntityValue *value = NULL;
+
+    const String temp_proc_name = String(temp_proc._name);
+ 
+    if(_containers.find(temp_proc_name) != _containers.end()) {
+        temp_container = _containers[temp_proc_name];
+    }
+    else {
+        temp_container = t->search_container(temp_proc_name);
+        _containers[temp_proc_name] = temp_container;
+    }
+
+    if(event_type == 0) {
+        Name name_temp(temp_marker._name, "");
+        t->define_event_type(name_temp, temp_container_type, extra_fields);
+        event_type = t->search_event_type(str_eventType);
+    }
+
+    value = t->search_entity_value(str_event, event_type);
+
+    if(temp_container == NULL && temp_proc._name != "0") {
+        // Creation of the container if not already done with beginProcess
+        handler_begin_process(trace, time, proc_id);
+        temp_proc = ParserDefinitionOTF::get_process_by_id(proc_id);
+        temp_container = t->search_container(temp_proc_name);
+    }
+
+    // Check if nothing is empty
+    if(event_type == 0) {
+        Error::set(Error::_UNKNOWN_EVENT_TYPE + n.to_string(), Error::_ERROR);
+    }
+    if(temp_container == 0 && temp_proc._name != "0") {
+        //Error::set(Error::_UNKNOWN_CONTAINER + temp_counter_group._name, Error::_ERROR);
+    }
+
+    t->new_event(d, event_type, temp_container, value, extra_fields);
+    return OTF_RETURN_OK;
 }
 
 int ParserEventOTF::handler_send_message (void *trace, uint64_t time, uint32_t sender, uint32_t receiver, uint32_t group, uint32_t type, uint32_t length, uint32_t source) {
@@ -561,11 +606,15 @@ int ParserEventOTF::read_events(OTF_Reader *reader) {
     return return_value;
 }
 
+void ParserEventOTF::read_markers(OTF_Reader *reader) {
+    OTF_Reader_readMarkers(reader, _handlers);
+}
+
 void ParserEventOTF::set_number_event_read_by_each_pass(OTF_Reader *reader, int number) {
     OTF_Reader_setRecordLimit(reader, number);
 }
 
-float ParserEventOTF::get_size_loaded() {
+float ParserEventOTF::get_percent_loaded() {
     if(ParserEventOTF::_max_time != ParserEventOTF::_min_time) {
         return (double)(ParserEventOTF::_cur_time - ParserEventOTF::_min_time)/(double)(ParserEventOTF::_max_time-ParserEventOTF::_min_time);
     }
