@@ -43,14 +43,26 @@
 /*!
  *\file Hook_event.cpp
  */
+
+
 #include <stack>
+#include <sstream>
+/* -- */
+#include <QMessageBox>
+#include <QMouseEvent>
+#include <QGLWidget>
 /* -- */
 #include "common/common.hpp"
-#include "common/info.hpp"
+#include "common/Info.hpp"
 /* -- */
-//#include "render/render.hpp"
+#include "interface/Interface.hpp"
+#include "core/Core.hpp"
+/* -- */
+#include "common/Message.hpp"
+/* -- */
+#include "render/Geometry.hpp"
+#include "render/Render_template.hpp"
 #include "render/Render_opengl.hpp"
-//#include "render/Geometry.hpp"
 #include "render/Hook_event.hpp"
 /* -- */
 using namespace std;
@@ -126,12 +138,14 @@ Hook_event::Hook_event(Render_opengl* render_instance, Core* core, QWidget *pare
 
     _ruler_distance = 5;
 
-    _mouse_x = 0;
-    _mouse_y = 0;
-    _new_mouse_x = 0;
-    _new_mouse_y = 0;
-    _mouse_pressed = false;
-    _key_scrolling = false;
+    _mouse_x                        = 0;
+    _mouse_y                        = 0;
+    _new_mouse_x                    = 0;
+    _new_mouse_y                    = 0;
+    _mouse_pressed                  = false;
+    _mouse_pressed_inside_container = false;
+    _mouse_pressed_inside_ruler     = false;
+    _key_scrolling                  = false;
 
     _minimum_distance_for_selection = 5;/* 5 pixels */
 
@@ -178,7 +192,10 @@ void Hook_event::mousePressEvent(QMouseEvent * event){
         /* Just a special case: if a zoom box is currently drawing, the right click is used
            to cancel the drawing */
         if ( true == _mouse_pressed){
-            _mouse_pressed = false;
+            _mouse_pressed                  = false;
+            _mouse_pressed_inside_container = false;
+            _mouse_pressed_inside_ruler     = false;
+
             updateRender();
             return;
         }
@@ -224,11 +241,26 @@ void Hook_event::mousePressEvent(QMouseEvent * event){
     }
 
     /* else, registered current X and Y mouse position to draw a zoom box */
-   
+
     _mouse_x = event->x();
     _new_mouse_x = _mouse_x;
+
+    /* user had clicked on the container */
+    if ( screen_to_render_x(_mouse_x) < _x_scale_container_state*Info::Render::width ){
+        _mouse_pressed_inside_container = true;
+        _mouse_x = render_to_screen_x(_x_scale_container_state*Info::Render::width);
+    }
+    
+
     _mouse_y = event->y();
     _new_mouse_y = _mouse_y;
+
+    /* user had clicked on the ruler */
+    if ( screen_to_render_y(_mouse_y) <= (_ruler_height+_ruler_y) ){
+        _mouse_pressed_inside_ruler = true;
+        _mouse_y = render_to_screen_y(_ruler_height+_ruler_y);
+    }
+
     _mouse_pressed = true;
 }
 
@@ -255,8 +287,15 @@ void Hook_event::mouseDoubleClickEvent ( QMouseEvent * event ){
 void Hook_event::mouseMoveEvent(QMouseEvent * event){
     if (!_mouse_pressed) return;
 
-    _new_mouse_x = event->x();
-    _new_mouse_y = event->y();
+    if (_mouse_pressed_inside_container)
+        _new_mouse_x = Info::Screen::width;
+    else
+        _new_mouse_x = event->x();
+
+    if (_mouse_pressed_inside_ruler)
+        _new_mouse_y = Info::Screen::height;
+    else
+        _new_mouse_y = event->y();
 
     updateRender();
 }
@@ -295,16 +334,40 @@ void Hook_event::mouseReleaseEvent(QMouseEvent * event){
         return;
 
     if (((_new_mouse_x-_mouse_x) < _minimum_distance_for_selection)
-        && ((_new_mouse_y-_mouse_y) < _minimum_distance_for_selection)){/* selection is too thin to draw a box. So, it must be a user click to displau entity information */
-        _mouse_pressed = false;
+        && ((_new_mouse_y-_mouse_y) < _minimum_distance_for_selection)){/* selection is too thin to draw a box. So, it must be a user click to display entity information */
+
+        _mouse_pressed                  = false;
+        _mouse_pressed_inside_container = false;
+        _mouse_pressed_inside_ruler     = false;
+        updateRender();
         return;/* escape */
     }
 
+    /* Thin box in the ruler: ignore */
+    if (_mouse_pressed_inside_ruler &&
+        ((_new_mouse_x-_mouse_x) < _minimum_distance_for_selection) ){
+   
+        _mouse_pressed              = false;    
+        _mouse_pressed_inside_ruler = false;
+        updateRender();
+        return;
+    }            
+
+    /* Thin box in the containers: ignore */
+    if (_mouse_pressed_inside_container &&
+        ((_new_mouse_y-_mouse_y) < _minimum_distance_for_selection) ){
+
+        _mouse_pressed                  = false;
+        _mouse_pressed_inside_container = false;
+        updateRender();
+        return;
+    }            
+
     /*
      * Now, user was drawing a box. Zoom in it!
      */
 
-    /* Now, register this position which will be used by a right clic */
+    /* Register this position which will be used by a right clic */
     selection_stack_buf.x_scale = _x_state_scale;
     selection_stack_buf.y_scale = _y_state_scale;
     selection_stack_buf.x_translate = _x_state_translate;
@@ -367,24 +430,23 @@ void Hook_event::apply_zoom_box(Element_pos x_min, Element_pos x_max, Element_po
      *
      * Work out the new selection middle position after applying the scale.
      */
-    //    x_scaled_middle_selection = (x_middle_selection + _x_state_translate - _default_entity_x_translate)*(Info::Screen::width/((x_max - x_min)));
-    x_scaled_middle_selection = (x_middle_selection + _x_state_translate - _default_entity_x_translate)*(Info::Screen::width/((x_max - x_min)));
-    y_scaled_middle_selection = (y_middle_selection + _y_state_translate - _ruler_height )*(Info::Screen::height/((y_max - y_min)));
+    x_scaled_middle_selection = (x_middle_selection + _x_state_translate - _default_entity_x_translate)*((Info::Screen::width*(1-_x_scale_container_state))/((x_max - x_min)));
+    y_scaled_middle_selection = (y_middle_selection + _y_state_translate - _ruler_height )*((Info::Screen::height - render_to_screen_y(_ruler_height))/((y_max - y_min)));
 
     /*
      * 2nd step:
      *
      * Work out the distance between the state origin and the render area middle (Info::Render::width/2).
      */
-    x_distance_between_state_origin_and_render_middle =  _x_state_translate - _default_entity_x_translate + Info::Render::width/2;
-    y_distance_between_state_origin_and_render_middle =  _y_state_translate - _ruler_height + Info::Render::height/2;
+    x_distance_between_state_origin_and_render_middle =  _x_state_translate + Info::Render::width*_x_scale_container_state - _default_entity_x_translate + Info::Render::width/2 - _default_entity_x_translate/2;
+    y_distance_between_state_origin_and_render_middle =  _y_state_translate + (Info::Render::height - _ruler_height)/2;
 
     /*
      * 3rd step:
      *
      * Translate entities.
      */
-    _x_state_translate +=  x_scaled_middle_selection -  x_distance_between_state_origin_and_render_middle;
+    _x_state_translate +=  x_scaled_middle_selection -  x_distance_between_state_origin_and_render_middle ;
     _y_state_translate +=  y_scaled_middle_selection -  y_distance_between_state_origin_and_render_middle;
 
 
@@ -394,15 +456,16 @@ void Hook_event::apply_zoom_box(Element_pos x_min, Element_pos x_max, Element_po
    
      /* NOTE: do not use replace_scale() because the translate will be also modified */
      int buf;
-
-     _x_state_scale *= Info::Screen::width/(x_max - x_min);
+     _x_state_scale *= (Info::Render::width*(1-_x_scale_container_state)) / screen_to_render_x(x_max - x_min);
+     
      buf = (int)(100*_x_state_scale);
      _core->launch_action(_core->_STATE_ZOOM_BOX_VALUE, &buf);
 
-
-    _y_state_scale *= Info::Screen::height/(y_max - y_min);
+    _y_state_scale *= (Info::Render::height - _ruler_height) / screen_to_render_y(y_max - y_min);
 
     _mouse_pressed = false;
+    _mouse_pressed_inside_container = false;
+    _mouse_pressed_inside_ruler     = false;
 
 
     /* Now, check if zoom is not too much */
@@ -412,10 +475,10 @@ void Hook_event::apply_zoom_box(Element_pos x_min, Element_pos x_max, Element_po
     
     if ( delta <=  zoom_limit ){/* if too much, cancel */
 
-        _x_state_scale *= (x_max - x_min) / Info::Screen::width;
-        _y_state_scale *= (y_max - y_min) / Info::Screen::height;
-        _x_state_translate -=  (x_scaled_middle_selection -  x_distance_between_state_origin_and_render_middle);
-        _y_state_translate -=  (y_scaled_middle_selection -  y_distance_between_state_origin_and_render_middle);
+        _x_state_scale     *= (x_max - x_min) / Info::Screen::width;
+        _y_state_scale     *= (y_max - y_min) / Info::Screen::height;
+        _x_state_translate -= (x_scaled_middle_selection -  x_distance_between_state_origin_and_render_middle);
+        _y_state_translate -= (y_scaled_middle_selection -  y_distance_between_state_origin_and_render_middle);
     }
 
     /* Update render */
