@@ -47,11 +47,6 @@
 #ifndef RENDER_HPP
 #define RENDER_HPP
 
-//#include "render/Render_opengl_dl.hpp"
-#include "render/Geometry.hpp"
-#include "render/Ruler.hpp"
-#include "common/info.hpp"
-
 /*!
  * \brief Structure used to store container information.
  */
@@ -217,7 +212,7 @@ public:
      * \brief Closes the container display list.
      */
     inline void end_draw_containers(){
-        float j = 0.6;
+        float j = 0.6f;
 
         //   std::cout << __FILE__ << " ~~ " << __LINE__ << " " << Info::Container::y_max << " " << Info::Container::y_min << std::endl;
         /* Before calling start_draw_container(), Info::Container::y_max should have a correct value */
