@@ -44,18 +44,23 @@
  *\file Render_opengl.cpp
  */
 
-#include "common/common.hpp"
-#include "common/info.hpp"
+#include <sstream>
+#include <cmath>
 /* -- */
-//#include "render/render.hpp"
-#include "render/Hook_event.hpp"
-
-#include "render/Render_opengl.hpp"
+#include <QFile> // For loading the wait image
+#include <QDate>
+#include <QTimer>
 /* -- */
 #include "interface/resource.hpp"
-//#include "interface/interface.hpp"
+#include "interface/Interface.hpp"
+/* -- */
+#include "common/common.hpp"
+#include "common/Info.hpp"
+#include "common/Message.hpp"
+/* -- */
+#include "render/Ruler.hpp"
+#include "render/Render_opengl.hpp"
 #include "render/Render.hpp"
-#include "core/Core.hpp"
 /* -- */
 
 using namespace std;
@@ -85,24 +90,24 @@ Render_opengl::Render_opengl(Core* core, QWidget *parent, const QGLFormat& forma
     _texts.clear();
 
     /* init the wait animation */
-    _wait_list    = 0;
-    _wait_angle   = 0.0f; /* begin with 0 rad angle */
-    _wait_spf     = DRAWING_TIMER_DEFAULT;/* DRAWING_TIMER_DEFAULT milliseconds per frame */
+    _wait_list      = 0;
+    _wait_angle     = 0.0f; /* begin with 0 rad angle */
+    _wait_timer     = NULL;
+    _wait_spf       = DRAWING_TIMER_DEFAULT;/* DRAWING_TIMER_DEFAULT milliseconds per frame */
+    /* _minimap_widget = NULL;
+       _minimap_image  = NULL;*/
 
-    _wait_timer   = NULL;
 #ifdef SPINNING_LOGO
     _wait_timer = new QTimer(this);
     connect(_wait_timer, SIGNAL(timeout()), _render_instance, SLOT(updateGL()));
     _wait_timer->start(_wait_spf);
 #endif // SPINNING_LOGO
 
- 
-
-     setAutoFillBackground(false);
+    setAutoFillBackground(false);
 }
 
 
-Render_opengl::~Render_opengl(){
+void Render_opengl::release(){
   /* Release the Rabbit and ruler lists */
     if (glIsList(_wait_list) == GL_TRUE)
         glDeleteLists(_wait_list, 1);
@@ -112,6 +117,8 @@ Render_opengl::~Render_opengl(){
         delete _wait_timer;
         _wait_timer=NULL;
     }
+  
+    //    release_minimap();
 }
 
 /***********************************
@@ -200,6 +207,7 @@ void  Render_opengl::paintGL(){
 	    glRotated(180.0, 1.0, 0.0, 0.0);
 	    
             glPushMatrix();
+
             {             
                 glTranslatef(0.0f, _ruler_y + _ruler_height -_y_state_translate,  _z_container);
                 glScalef(_x_scale_container_state/0.20, _y_state_scale, 1.0);
@@ -208,6 +216,7 @@ void  Render_opengl::paintGL(){
                     *Message::get_instance() << tr("ERROR LIST not exist for containers.").toStdString() << Message::ende;
                 else
                     glCallList(_list_containers);
+
             }
             glPopMatrix();
 
@@ -253,10 +262,11 @@ void  Render_opengl::paintGL(){
                
  
 
-                old_x = _mouse_x*(Info::Render::width/Info::Screen::width);
-                old_y =  Info::Render::height - _mouse_y*(Info::Render::height/Info::Screen::height);
-                new_x = _new_mouse_x*(Info::Render::width/Info::Screen::width);
-                new_y =  Info::Render::height - _new_mouse_y*(Info::Render::height/Info::Screen::height);
+                old_x = screen_to_render_x(_mouse_x);
+                old_y = Info::Render::height - screen_to_render_y(_mouse_y);
+
+                new_x = screen_to_render_x(_new_mouse_x);
+                new_y = Info::Render::height -  screen_to_render_y(_new_mouse_y);
 
 #ifdef DEBUG_MODE_RENDER_OPENGL
                
@@ -311,68 +321,76 @@ void  Render_opengl::paintGL(){
 
     /* Draw container text */
     const unsigned int texts_size = _texts.size();
-    for (unsigned int i=0 ; i<texts_size ; i++){
-        const QFontMetrics metric(arial_font);
-        const QString text_elided = metric.elidedText(_texts[i].value.c_str(), Qt::ElideRight, _x_scale_container_state*Info::Screen::width/3.);
-        
-        if (trace_to_render_y(_texts[i].y) + 0.5 < 9) continue;/* Do not display text if it is on the ruler area */
-
-        renderText ( render_to_screen_x(_texts[i].x * _x_scale_container_state/0.20),
-                     render_to_screen_y( trace_to_render_y(_texts[i].y) + 0.5),
-                     text_elided,
-                     arial_font);
-    }
+        for (unsigned int i=0 ; i<texts_size ; i++){
+            const QFontMetrics metric(arial_font);
+            const QString text_elided = metric.elidedText(_texts[i].value.c_str(), Qt::ElideRight, _x_scale_container_state*Info::Screen::width/(Info::Trace::depth+1.));
+            
+            if (trace_to_render_y(_texts[i].y) + 0.5 < 9) continue;/* Do not display text if it is on the ruler area */
+            
+            renderText ( render_to_screen_x(_texts[i].x * _x_scale_container_state/0.20),
+                         render_to_screen_y( trace_to_render_y(_texts[i].y) + 0.5),
+                         text_elided,
+                         arial_font);
+        }
+  
 
     /* Draw ruler text */
-    std::ostringstream buf_txt;
-    Element_pos graduation_diff;
-    Element_pos coeff_prefix;
-    graduation_diff = Ruler::get_graduation_diff(Info::Render::_x_min_visible, Info::Render::_x_max_visible);
-    coeff_prefix    = Ruler::get_coeff_for_common_prefix(Info::Render::_x_min_visible, Info::Render::_x_max_visible);
-
-    arial_font.setPointSize(14);
-
-    buf_txt.str("");/* flush the buffer */
-    buf_txt << "min: " << (double)Info::Render::_x_min_visible;
-
-    renderText (render_to_screen_x(trace_to_render_x(Info::Render::_x_min_visible)),
-                render_to_screen_y(3),
-                buf_txt.str().c_str(),
-                arial_font);
-
-    buf_txt.str("");/* flush the buffer */
-    buf_txt << "max: " << (double)Info::Render::_x_max_visible;
-
-    renderText (render_to_screen_x(trace_to_render_x(Info::Render::_x_max_visible))-130,
-                render_to_screen_y(3),
-                buf_txt.str().c_str(),
-                arial_font);
-
-    buf_txt.str("");
-    buf_txt << Ruler::get_common_part_string(Info::Render::_x_min_visible, coeff_prefix) << "--";
-
-    renderText (render_to_screen_x(
-                                   trace_to_render_x(
-                                                     (Info::Render::_x_min_visible + 
-                                                      Info::Render::_x_max_visible) / 2)), 
-                render_to_screen_y(3),
-                buf_txt.str().c_str(),
-                arial_font);
-
-    arial_font.setPointSize(10);
-
-    for (Element_pos i = Info::Render::_x_min_visible ; 
-         i < Info::Render::_x_max_visible ; 
-         i+=graduation_diff){
-
+         std::ostringstream buf_txt;
+        Element_pos graduation_diff;
+        Element_pos coeff_prefix;
+        graduation_diff = Ruler::get_graduation_diff(Info::Render::_x_min_visible, Info::Render::_x_max_visible);
+        coeff_prefix    = Ruler::get_coeff_for_common_prefix(Info::Render::_x_min_visible, Info::Render::_x_max_visible);
+        
+        arial_font.setPointSize(14);
+        
         buf_txt.str("");/* flush the buffer */
-        buf_txt << Ruler::get_variable_part(i, coeff_prefix, 2);
-   
-        renderText (render_to_screen_x(trace_to_render_x(i)+1),
-                    render_to_screen_y(8),
+        buf_txt << "min: " << (double)Info::Render::_x_min_visible;
+        
+        renderText (render_to_screen_x(trace_to_render_x(Info::Render::_x_min_visible)),
+                    render_to_screen_y(3),
                     buf_txt.str().c_str(),
                     arial_font);
-    }      
+        
+        buf_txt.str("");/* flush the buffer */
+        buf_txt << "max: " << (double)Info::Render::_x_max_visible;
+        
+        renderText (render_to_screen_x(trace_to_render_x(Info::Render::_x_max_visible))-130,
+                    render_to_screen_y(3),
+                    buf_txt.str().c_str(),
+                    arial_font);
+        
+        buf_txt.str("");
+        buf_txt << Ruler::get_common_part_string(Info::Render::_x_min_visible, coeff_prefix) << "--";
+        
+        renderText (render_to_screen_x(
+                                       trace_to_render_x(
+                                                         (Info::Render::_x_min_visible + 
+                                                          Info::Render::_x_max_visible) / 2)), 
+                    render_to_screen_y(3),
+                    buf_txt.str().c_str(),
+                    arial_font);
+        
+        arial_font.setPointSize(10);
+        
+        for (Element_pos i = Info::Render::_x_min_visible ; 
+             i < Info::Render::_x_max_visible ; 
+             i+=graduation_diff){
+            
+            buf_txt.str("");/* flush the buffer */
+            buf_txt << Ruler::get_variable_part(i, coeff_prefix, 2);
+            
+            renderText (render_to_screen_x(trace_to_render_x(i)+1),
+                        render_to_screen_y(8),
+                        buf_txt.str().c_str(),
+                        arial_font);
+        }      
+       
+         //  update_minimap();
+        _minimap.update( ( ( (_x_state_translate - _default_entity_x_translate)/ (Info::Render::width*_x_state_scale) ) +  (_default_entity_x_translate/Info::Render::width) )*_minimap.width(),
+                         ( (_y_state_translate - _ruler_height) / (Info::Render::height*_y_state_scale) + (_ruler_height/Info::Render::height) )*_minimap.height(),
+                         _minimap.width()/_x_state_scale,
+                         _minimap.height()/_y_state_scale
+                         );
 }
 
 
@@ -399,12 +417,9 @@ bool Render_opengl::build(){
     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
 
+
     replace_scale(1);/* for states scaling */
-    _x_state_translate = 0;/* for states translation */
-    _y_state_scale = 1;/* for states scaling */
-    _y_state_translate = 0;/* for states translation */
-    _x_scroll_pos = 0;/* horizontal bar placed on 0 */
-    _y_scroll_pos = 0;/* vertical bar placed on 0 */
+    init_geometry();/* Initialize geometry */
 
 #ifdef SPINNING_LOGO
     if (_wait_timer != NULL){
@@ -418,6 +433,12 @@ bool Render_opengl::build(){
     if (NULL == _render_instance)
         return true;
 
+
+    updateGL();
+    QImage buf = grabFrameBuffer(true);
+    _minimap.init(buf);
+    updateGL();
+
     return true;//_render_instance->display_build();
 }
 
@@ -452,27 +473,7 @@ bool Render_opengl::unbuild(){
     _state = DRAWING_STATE_WAITING;/* change the drawing state */
 
     _wait_angle=0.0f;/* begin with 0 rad angle */
-
-    /* init measurement attributes */    
-    /*    Info::Container::x_max = 0;
-    Info::Container::y_max = 0;
-    Info::Container::x_min = 0;
-    Info::Container::y_min = 0;
-
-    Info::Entity::x_max = 0;
-    Info::Entity::x_min = 0;*/
-    //    _state_x_max = 0;
-    _state_y_max = 0;
-    //    _state_x_min = 0;
-    _state_y_min = 0;
-
-    /* TODO: call method which release Geometry attribute values */
-    _x_state_scale     = 1;/* for states scaling */
-    _x_state_translate = 0;/* for states translation */
-    _y_state_scale     = 1;/* for states scaling */
-    _y_state_translate = 0;/* for states translation */
-    _x_scroll_pos      = 0;/* horizontal bar placed on 0 */
-    _y_scroll_pos      = 0;
+    
 
     /* clear lists to store container texts */
     _text_pos.clear();
@@ -481,12 +482,6 @@ bool Render_opengl::unbuild(){
     _arrows.clear();
     _circles.clear();
     _texts.clear();
-
-    /* clear the event vector */
-    // _events.clear();
-
-    /* clear the arrow vector */
-    // _arrows.clear();
     
     /* empty the selection stack */
     while(false == _previous_selection.empty())
@@ -505,6 +500,9 @@ bool Render_opengl::unbuild(){
         _wait_timer->start(_wait_spf);
     }
 #endif // SPINNING_LOGO
+    _minimap.release();
+
+    init_geometry();/* Initialize geometry */
 
     return true;//_render_instance->display_unbuild();
 }
@@ -584,9 +582,6 @@ GLuint Render_opengl::draw_wait() {
 }
 
 
-
-
-
 void Render_opengl::call_ruler(){
     Element_pos graduation_diff;
     Element_pos coeff_prefix;
@@ -600,13 +595,23 @@ void Render_opengl::call_ruler(){
     
     set_color(1.0, 1.0, 1.0);
     
-    for (Element_pos i = Info::Render::_x_min_visible ; i < Info::Render::_x_max_visible ; i+=graduation_diff){
+    for (Element_pos i = Info::Render::_x_min_visible ;
+         i < Info::Render::_x_max_visible ;
+         i+=graduation_diff){
+
         const Element_pos grad_div_by_5 = graduation_diff/5;
         for (Element_pos j = (i+grad_div_by_5) ; j<(i+graduation_diff) ; j+= grad_div_by_5){
+
+            if (j>=Info::Render::_x_max_visible)
+                break;
+
             draw_line( trace_to_render_x(j) + offset_x, offset_y  ,
                        trace_to_render_x(j) + offset_x, 2+offset_y, _z_ruler);
         }
 
+        if (i>=Info::Render::_x_max_visible)
+            break;
+   
         draw_line( trace_to_render_x(i) + offset_x, offset_y-1  ,
                    trace_to_render_x(i) + offset_x, 4+offset_y, _z_ruler);
     }
@@ -647,7 +652,7 @@ void Render_opengl::draw_text(const Element_pos x, const Element_pos y, const El
     if (_draw_state) message << "Fatal error! Cannot put text in state lists!" << Message::ende;
 
         Container_text_ buf;
-
+        
         buf.x     = x;
         buf.y     = render_to_trace_y(y);/* Cancel previous transformation. */
         buf.value = s;
@@ -666,13 +671,17 @@ void Render_opengl::draw_quad(Element_pos x, Element_pos y, Element_pos /*z*/, E
 
     if (!_draw_container)
         offset_x = -_default_entity_x_translate;
-    
-     glBegin(GL_QUADS);
+   
+
+    glBegin(GL_QUADS);
     {
         glColor3d(_red, _green, _blue);
+      
         glVertex2d(x + offset_x    , y + offset_y);
         glVertex2d(x + offset_x    , y + h + offset_y);
-        glColor3d(_red/1.5         , _green/1.5  , _blue/1.5);
+
+        glColor3d(_red/1.5, _green/1.5, _blue/1.5);
+      
         glVertex2d(x + w + offset_x, y + h + offset_y);
         glVertex2d(x + w + offset_x, y + offset_y);
     }
@@ -688,10 +697,10 @@ void Render_opengl::draw_line(Element_pos x1, Element_pos y1, Element_pos x2, El
     const Element_pos offset_x = -_default_entity_x_translate;
     const Element_pos offset_y = -_ruler_y - _ruler_height;
 
-
     if (_draw_ruler || _draw_arrow || _draw_event) return;/* do not draw line for ruler or for arrow or for event */
-    if (_draw_state) message << "Fatal error! Cannot put line in state lists!" << Message::ende;
 
+    if (_draw_state) message << "Fatal error! Cannot put line in state lists!" << Message::ende;
+   
    
     glBegin(GL_LINES);
     {
@@ -713,7 +722,9 @@ void Render_opengl::start_draw(){
     _draw_ruler     = false;
     _draw_arrow     = false;
     _draw_event     = false;
-   
+ 
+    init_geometry();/* Initialize geometry */
+  
 }
 
 void Render_opengl::start_draw_containers(){
@@ -970,3 +981,104 @@ void Render_opengl::draw_stored_circles(){
     }
 }
     
+// void Render_opengl::create_minimap(const int width, const int height){
+
+//     _minimap_widget = new QLabel;
+//     _minimap_image = new QImage;
+
+//     if (!_minimap_widget || !_minimap_image){
+//         message << "Cannot allocate memory for minimap" << Message::endw;
+//         return;
+//     }
+
+//     paintGL();
+
+//     *_minimap_image = grabFrameBuffer(true);
+    
+//     //    _default_entity_x_translate = 20; /* TODO: use constant value define in config file */ 
+//     /*_ruler_height               = 8.5;
+//       _ruler_y                    = 0.0;*/
+    
+//     //  *_minimap_image = _minimap_image->scaled(QSize(width, height), Qt::IgnoreAspectRatio);
+//     _minimap_widget->resize(width, height);
+//     _minimap_widget->hide();/* Do not display it */
+//     //  update_minimap();
+// }
+
+// void Render_opengl::update_minimap(){
+//     QPixmap buf;
+//     static int i = 0;
+
+//     if (!_minimap_image)
+//         return;
+
+//     if (!_minimap_widget)
+//         return;
+
+//     buf = QPixmap::fromImage(*_minimap_image);
+
+//     if (buf.isNull())
+//         return;
+
+//     QPainter painter;
+//     QPen pen;
+//     painter.begin(&buf);
+
+//     pen.setColor(Qt::red);
+//     pen.setWidth(4);
+
+//     painter.setPen(pen);
+//     painter.setBrush(QBrush(QColor::fromRgbF(1, 1, 0, 0.5)));
+
+//     painter.drawRect ( ( ( (_x_state_translate - _default_entity_x_translate)/ (Info::Render::width*_x_state_scale) ) +  (_default_entity_x_translate/Info::Render::width) )*_minimap_image->width(),
+//                       //render_to_screen_x(trace_to_render_x(Info::Render::_x_min_visible))*200/Info::Screen::width,
+//                        ( (_y_state_translate - _ruler_height) / (Info::Render::height*_y_state_scale) + (_ruler_height/Info::Render::height))*_minimap_image->height(),
+//                        _minimap_image->width()/_x_state_scale,//render_to_screen_x(trace_to_render_x(Info::Render::_x_max_visible - Info::Render::_x_min_visible)),
+//                        _minimap_image->height()/_y_state_scale);
+//     painter.end();
+    
+
+//     _minimap_widget->setPixmap(buf);
+//     _minimap_widget->setScaledContents(true);
+
+//     if (!_minimap_widget->pixmap())
+//         message << tr("No Pixmap set for the minimap").toStdString() << Message::endw;
+// }
+
+
+void Render_opengl::show_minimap(){
+
+    //if (state)
+        _minimap.show();
+        /*    else
+              _minimap.hide();*/
+    // if (!_minimap_widget) return;
+
+    // if (state)
+    //     _minimap_widget->show();
+    // else
+    //     _minimap_widget->hide();
+}
+
+
+// void Render_opengl::release_minimap(){
+    
+//     if (_minimap_image){
+//         delete _minimap_image;
+//         _minimap_image = NULL;
+//     }
+
+//     if (_minimap_widget)
+//         _minimap_widget->close();
+
+//     /* Qt will release automatically _minimap_widget */
+// }
+
+
+void Render_opengl::clear_arrow (){
+    _arrows.clear();
+}
+
+void Render_opengl::clear_text (){
+    _texts.clear();
+}
