@@ -47,17 +47,27 @@
 #ifndef RENDER_OPENGL_HPP
 #define RENDER_OPENGL_HPP
 
+/* For moc compilation */
+#include <stack>
+#include <cmath>
+/* -- */
+#include <QObject>
 #include <QGLWidget>
-
-#include "core/Core.hpp"
+#include <QLabel>
+#include <QPainter>
+/* -- */
+#include "common/common.hpp"
+#include "common/Info.hpp"
+/* -- */
+#include "render/Geometry.hpp"
 #include "render/Hook_event.hpp"
 #include "render/Render_template.hpp"
+#include "render/Minimap.hpp"
+#include "render/Ruler.hpp"
+#include "render/Render.hpp"
 
-class Render_opengl;
-class Interface_console;
 struct Container_text_;
 
-
 /*!
  * \brief Structure used to store arrow information.
  */
@@ -115,6 +125,10 @@ protected:
      */
      GLuint _wait_list;
 
+    /*!
+     * \brief The minimap.
+     */
+    Minimap _minimap;
 
     /*!
      * \brief The container GLu list.
@@ -177,6 +191,34 @@ protected:
      */
     QTimer* _wait_timer;
 
+
+    /*!
+     * \brief QLabel to display minimap.
+     */
+    //  QLabel* _minimap_widget;
+
+    /*!
+     * \brief QImage to store the trace thumbnail.
+     */
+    // QImage* _minimap_image;
+
+    /*!
+     * \brief Create a minimap from the current render.
+     * \param width The desired width for the image.
+     * \param height The desired height for the image.
+     */
+    // void create_minimap(const int width, const int height);
+
+    /*!
+     * \brief Update the minimap according to the current viewport.
+     */
+    //   void update_minimap();
+
+    /*!
+     * \brief Release data for minimap.
+     */
+    //  void release_minimap();
+
 public:
 
     /***********************************
@@ -191,9 +233,9 @@ public:
     Render_opengl(Core* core, QWidget *parent, const QGLFormat& format);
 
     /*!
-     * \brief The destructor
+     * \brief The destructor (~Render_opengl will never be called, so use another method to release memory)
      */
-    virtual ~Render_opengl();
+    void release();
 
     /*!
      * \brief Set Statistics and Informations about input trace
@@ -562,6 +604,16 @@ public:
     void draw_stored_circles();
 
     void draw_stored_texts();
+
+    /*!
+     * \brief Show the minimap.
+     */
+    void show_minimap();
+
+    /*! Function that only delete the arrows of the screen */
+    void clear_arrow ();
+    /*! Function that only delete the text of the screen */
+    void clear_text ();
 };
 
 
