@@ -194,32 +194,32 @@ public:
     void build(T* draw_object, Trace* trace) {
         double zoom = trace->get_filter();
         Interval * interval;// = trace->get_interval_constrained();
-        std::vector<const Container *> *container = trace->get_selected_container();
+	std::vector<const Container *> *container = trace->get_selected_container();
         if(container == NULL ||
 
-           container->empty()){
+            container->empty()){
             container = new std::vector<const Container *>();
             
             trace->set_filter(0);
 
             std::stack<Container *> containers;
-            for (std::list<Container *>::const_iterator i = trace->get_root_containers()->begin();
+            for (Container::VectorIt i = trace->get_root_containers()->begin();
                  i != trace->get_root_containers()->end();
                  i++){
                 containers.push(*i);
                 container->push_back(*i);
             }
-            add(container,&containers);
+            add(container, &containers);
 
         }
         //Adding the parent containers if not added yet
         else{
             bool ended = false;
-            std::vector<const Container *> store;
+	    std::vector<const Container *> store;
             store.clear();
             while (!ended){
                 ended = true;
-                for(std::vector<const Container*>::iterator it = container->begin() ;
+                for(std::vector<const Container *>::const_iterator it = container->begin() ;
                     it != container->end();
                     it ++){
 
@@ -230,7 +230,7 @@ public:
                         }
                 }//end for
                 //TODO Warning the same container should be added more than 1 time
-                for(std::vector<const Container*>::iterator it = store.begin() ;
+                for(std::vector<const Container *>::const_iterator it = store.begin() ;
                     it != store.end();
                     it ++){
                     if( !is_in_set(*it,container))
@@ -259,20 +259,20 @@ public:
      *
      **********************************/
     /*
-     * \fn browse_container_tree(T* draw_object, Trace* trace, std::vector<Container *> container)
+     * \fn browse_container_tree(T* draw_object, Trace* trace, Container::Vector container)
      * \brief Function that browses the containers of the trace argument that are in the set container and make them painted with a T object
      */
     template<class T>
-    inline void browse_container_tree(T* draw_object, Trace* trace, std::vector< const Container *> *container){
+    inline void browse_container_tree(T* draw_object, Trace* trace, std::vector<const Container *> *container){
         /*** Drawing containers ***/
         int position = 0;
         
-        const std::list<Container *> *root_containers = trace->get_root_containers();
+        const Container::Vector *root_containers = trace->get_root_containers();
         if (root_containers->empty()) {
             *Message::get_instance() << QObject::tr("There is no container. The trace can not be drawn.").toStdString() << Message::endw;
         }
         else
-            for (std::list<Container *>::const_iterator i = root_containers->begin();
+            for (Container::VectorIt i = root_containers->begin();
                 i != root_containers->end();
                 i++)
                 // Use the fact a container is selected implies all his ancestors are
@@ -282,17 +282,17 @@ public:
 
 
     /*
-     * \fn browse_container(T* draw_object, const Container *container, int position, int depth, std::vector<Container *> set_container)
+     * \fn browse_container(T* draw_object, const Container *container, int position, int depth, Container::Vector set_container)
      * \brief Recursive function that browse a container to draw it if it is in the set of container with a T painting object in position, knowing the current depth in the tree
      */
 
     template<class T>
-    int browse_container(T* draw_object, const Container *container, int position, int depth, std::vector< const Container *> *set_container) {
+    int browse_container(T* draw_object, const Container *container, int position, int depth, std::vector<const Container *> *set_container) {
         int size = 0;
         
         // Draw children
-        const std::list<Container *> *children = container->get_children();
-        for (std::list<Container *>::const_iterator i = children->begin();
+        const Container::Vector *children = container->get_children();
+        for (Container::VectorIt i = children->begin();
             i != children->end();
             i++) {
             if(is_in_set(*i,set_container))
@@ -350,9 +350,9 @@ public:
     inline void browse_entities(T* draw_object, double zoom, Interval* interval, std::vector<const Container *> *set_container) {
         //////////////////////////////////////////////////////////
         const Container *container;
-        BinaryTree<StateChange> *state_tree;
-        BinaryTree<Event> *event_tree;
-        const std::list<Link *> *link_list;
+        StateChange::Tree *state_tree;
+        Event::Tree *event_tree;
+        const Link::Vector *link_list;
         Link *link;
         const std::map<VariableType *, Variable *> *variable_map;
         Variable *var;
@@ -408,7 +408,7 @@ public:
 	        
 	        // Browse links
 	        link_list = container->get_links();
-                for (std::list<Link *>::const_iterator it = link_list->begin();
+                for (Link::VectorIt it = link_list->begin();
                      it != link_list->end();
                      it++) {
                     
@@ -530,9 +530,9 @@ public:
         
         // find container needs to know the position of each container
         double yr = y;
-        const std::list<Container *> *root_containers = trace->get_root_containers();
+        const Container::Vector *root_containers = trace->get_root_containers();
         if (!root_containers->empty())
-            for (std::list<Container *>::const_iterator i = root_containers->begin();
+            for (Container::VectorIt i = root_containers->begin();
                 i != root_containers->end();
                 i++)
                 if ((container = search_container_by_position(*i, yr)))
@@ -544,7 +544,7 @@ public:
          
         // Calculate the container positions
         int position = 0;
-        for (std::list<Container *>::const_iterator i = root_containers->begin();
+        for (Container::VectorIt i = root_containers->begin();
                 i != root_containers->end();
                 i++)
                 position += calc_container_positions(*i, position);
@@ -644,8 +644,8 @@ public:
     const Container *search_container_by_position(const Container *container, double &y) {
         const Container *result;
         // Search if the result is a descendant
-        const std::list<Container *> *children = container->get_children();
-        for (std::list<Container *>::const_iterator i = children->begin();
+        const Container::Vector *children = container->get_children();
+        for (Container::VectorIt i = children->begin();
             i != children->end();
             i++) {
             if ((result = search_container_by_position(*i, y)) != NULL)
@@ -681,8 +681,8 @@ public:
         int size = 0;
         
         // Draw children
-        const std::list<Container *> *children = container->get_children();
-        for (std::list<Container *>::const_iterator i = children->begin();
+        const Container::Vector *children = container->get_children();
+        for (Container::VectorIt i = children->begin();
             i != children->end();
             i++) {
             size += calc_container_positions((*i), position+size);
@@ -708,7 +708,7 @@ public:
      * \brief Tries to find a link passing by x and y in the container
      */
     const Link* get_link(const Container *container, Element_pos x, Element_pos y, Element_pos d) {
-        const std::list<Link *> *link_list;
+        const Link::Vector *link_list;
         Link *link;
         double a, b, c; // Equation: ax + by + c = 0
         double x1, x2, y1, y2;
@@ -718,7 +718,7 @@ public:
         
         // Browse links
         link_list = container->get_links();
-        for (std::list<Link *>::const_iterator it = link_list->begin();
+        for (Link::VectorIt it = link_list->begin();
              it != link_list->end();
              it++) {
             
@@ -796,11 +796,7 @@ public:
     }
 
 
-
-
-
-
-    bool link_is_in_set(Link * link, std::vector<const Container *> *set_container, Interval * interval ){
+  bool link_is_in_set(Link * link, std::vector<const Container *> *set_container, Interval * interval ){
         const Container * src = link->get_source();
         const Container * dest = link->get_destination();
         for(unsigned int i = 0 ; i < set_container->size() ; i++){
@@ -816,7 +812,7 @@ public:
         return false;
     }
 
-    bool is_in_set(const Container * c,std::vector<const Container *>*set_container){
+  bool is_in_set(const Container * c, std::vector<const Container *> *set_container){
         if(!c || !set_container)
             return false;
         for(unsigned int i = 0 ; i<set_container->size() ; i++ )
@@ -827,13 +823,13 @@ public:
 
 
 
-    void add( std::vector<const Container *>* container,std::stack<Container *> *containers){
+  void add( std::vector<const Container *> *container,std::stack<Container *> *containers){
  
         
         while (!containers->empty()) {
             const Container * c = containers->top();
             containers->pop();
-            for (std::list<Container *>::const_iterator i = c->get_children()->begin();
+            for (Container::VectorIt i = c->get_children()->begin();
                  i != c->get_children()->end();
                  i++){
                 containers->push(*i);
