@@ -40,23 +40,35 @@
 **        - VUCHENER Clément 
 **
 */
-
+#include <iostream>
 #include <string>
 #include <map>
 #include <list>
 #include <vector>
 #include <sstream>
 #include <stack>
+#include <algorithm>
+/* -- */
+#include "interface/Interface.hpp"
 /* -- */
 #include "common/Message.hpp"
+#include "common/Session.hpp"
+/* -- */
+#include "render/Palette.hpp"
 /* -- */
 #include "trace/values/Values.hpp"
-#include "trace/values/Date.hpp"
+#include "trace/tree/Interval.hpp"
+#include "trace/tree/Node.hpp"
+#include "trace/tree/BinaryTree.hpp"
 #include "trace/EntityValue.hpp"
 #include "trace/EntityTypes.hpp"
 #include "trace/Entitys.hpp"
 #include "trace/Trace.hpp"
 /* -- */
+#include "common/common.hpp"
+#include "common/Info.hpp" /* Dirty, should be remove */
+#include "common/Message.hpp"
+/* -- */
 #include "trace/tree/Interval.hpp"
 /* -- */
 using namespace std;
@@ -66,19 +78,40 @@ Trace::Trace(): _max_date(0.0) {
     _selected_container = NULL;
     _filter = 0;
     _interval_constrained = NULL;
+    _depth = 0;
+
+#ifndef WITHOUT_QT
+    if(Session::get_use_palette())
+        _palette = Session::get_palette(Session::get_current_palette());
+    else
+#endif
+        _palette = NULL;
 }
 
+template <class T>
+void MyDelete(T *ptr){
+  delete ptr;
+};
+
+
 Trace::~Trace() {
+    //delete _palette;
+
     if(_interval_constrained != NULL) {
         delete _interval_constrained;
         _interval_constrained = NULL;
     }
 
     // Delete containers
-    while (!_root_containers.empty()){
-        delete _root_containers.front();
-        _root_containers.pop_front();
-    }
+    for_each(_root_containers.begin(), 
+	     _root_containers.end(), 
+	     MyDelete<Container>);
+    _root_containers.clear();
+    _root_containers.resize(0);
+    // while (!_root_containers.empty()){
+    //     delete _root_containers.front();
+    //     _root_containers.pop_front();
+    // }
     
     // Delete container types
     while (!_root_container_types.empty()){
@@ -111,15 +144,15 @@ Trace::~Trace() {
     }
 }
 
-static void delete_opt(map<std::string, Value *> &opt) {
-    for (map<std::string, Value *>::iterator i = opt.begin();
+static void delete_opt(map<string, Value *> &opt) {
+    for (map<string, Value *>::iterator i = opt.begin();
         i != opt.end();
         i++) {
         delete (*i).second;
     }
 }
 
-void Trace::define_container_type(Name &name, ContainerType *parent, map<std::string, Value *> &opt) {
+void Trace::define_container_type(Name &name, ContainerType *parent, map<string, Value *> &opt) {
     ContainerType *type = new ContainerType(name, parent);
     if (parent)
         parent->add_child(type);
@@ -130,16 +163,25 @@ void Trace::define_container_type(Name &name, ContainerType *parent, map<std::st
     delete_opt(opt);
 }
 
-void Trace::create_container(Date &time, Name &name, ContainerType *type, Container *parent, map<std::string, Value *> &opt) {
-    if (!type)
+void Trace::create_container(Date &time, Name &name, ContainerType *type, Container *parent, map<string, Value *> &opt) {
+    if (!type) {
+	vite_error("Trace::get_container : Type undefined");
         return;
+    }
     
     Container *cont = new Container(name, time, type, parent, opt);
-    if (parent)
+    if (parent){
         parent->add_child(cont);
-    else
+    } else {
         _root_containers.push_back(cont);
-    
+    }
+
+    /* Update the trace depth */
+    if (cont->get_depth() > _depth){
+            _depth=cont->get_depth();
+            Info::Trace::depth=_depth; /* Dirty method. Should be remove */
+    }
+
     if (time > _max_date)
         _max_date = time;
         
@@ -147,7 +189,7 @@ void Trace::create_container(Date &time, Name &name, ContainerType *type, Contai
     //delete_opt(opt);
 }
 
-void Trace::destroy_container(Date &time, Container *cont, ContainerType *type, map<std::string, Value *> &opt) {
+void Trace::destroy_container(Date &time, Container *cont, ContainerType *type, map<string, Value *> &opt) {
     if (cont && type)
         cont->destroy(time);
     
@@ -158,32 +200,43 @@ void Trace::destroy_container(Date &time, Container *cont, ContainerType *type,
     delete_opt(opt);
 }
 
-void Trace::define_event_type(Name &name, ContainerType *container_type, map<std::string, Value *> &opt) {
+void Trace::define_event_type(Name &name, ContainerType *container_type, map<string, Value *> &opt) {
     if (container_type)
         _event_types.push_back(new EventType(name, container_type, opt));
 }
 
-void Trace::define_state_type(Name &name, ContainerType *container_type, map<std::string, Value *> &opt) {
+void Trace::define_state_type(Name &name, ContainerType *container_type, map<string, Value *> &opt) {
     if (container_type)
         _state_types.push_back(new StateType(name, container_type, opt));
 }
 
-void Trace::define_variable_type(Name &name, ContainerType *container_type, map<std::string, Value *> &opt) {
+void Trace::define_variable_type(Name &name, ContainerType *container_type, map<string, Value *> &opt) {
     if (container_type)
         _variable_types.push_back(new VariableType(name, container_type, opt));
 }
 
-void Trace::define_link_type(Name &name, ContainerType *ancestor, ContainerType *source, ContainerType *destination, map<std::string, Value *> &opt) {
+void Trace::define_link_type(Name &name, ContainerType *ancestor, ContainerType *source, ContainerType *destination, map<string, Value *> &opt) {
     if (ancestor && source && destination)
         _link_types.push_back(new LinkType(name, ancestor, source, destination, opt));
 }
 
-void Trace::define_entity_value(Name &name, EntityType *entity_type, map<std::string, Value *> &opt) {
-    if (entity_type)
+void Trace::define_entity_value(Name &name, EntityType *entity_type, map<string, Value *> &opt) {
+    if (entity_type) {
+        if(_palette) {
+            Color *c = _palette->get_color(name.to_string());
+            if(c != NULL) { // We set the palette color first
+                if(opt.find("Color") != opt.end()) {
+                    delete opt["Color"];
+                }
+                opt["Color"] = new Color(*c);
+            }
+        }
+
         entity_type->add_value(new EntityValue(name, entity_type, opt));
+    }
 }
 
-void Trace::set_state(Date &time, StateType *type, Container *container, EntityValue *value, map<std::string, Value *> &opt) {
+void Trace::set_state(Date &time, StateType *type, Container *container, EntityValue *value, map<string, Value *> &opt) {
     if (container && type)
         container->set_state(time, type, value, opt);
     
@@ -191,7 +244,7 @@ void Trace::set_state(Date &time, StateType *type, Container *container, EntityV
         _max_date = time;
 }
 
-void Trace::push_state(Date &time, StateType *type, Container *container, EntityValue *value, map<std::string, Value *> &opt) {
+void Trace::push_state(Date &time, StateType *type, Container *container, EntityValue *value, map<string, Value *> &opt) {
     if (container && type)
         container->push_state(time, type, value, opt);
     
@@ -199,7 +252,7 @@ void Trace::push_state(Date &time, StateType *type, Container *container, Entity
         _max_date = time;
 }
 
-void Trace::pop_state(Date &time, StateType *type, Container *container, map<std::string, Value *> &opt) {
+void Trace::pop_state(Date &time, StateType *type, Container *container, map<string, Value *> &opt) {
     if (container && type)
         container->pop_state(time);
     
@@ -210,7 +263,7 @@ void Trace::pop_state(Date &time, StateType *type, Container *container, map<std
     delete_opt(opt);
 }
 
-void Trace::new_event(Date &time, EventType *type, Container *container, EntityValue *value, map<std::string, Value *> &opt) {
+void Trace::new_event(Date &time, EventType *type, Container *container, EntityValue *value, map<string, Value *> &opt) {
     if (container && type)
         container->new_event(time, type, value, opt);
     
@@ -218,7 +271,7 @@ void Trace::new_event(Date &time, EventType *type, Container *container, EntityV
         _max_date = time;
 }
 
-void Trace::set_variable(Date &time, VariableType *type, Container *container, Double value, map<std::string, Value *> &opt) {
+void Trace::set_variable(Date &time, VariableType *type, Container *container, Double value, map<string, Value *> &opt) {
     if (container && type)
         container->set_variable(time, type, value);
     
@@ -229,7 +282,7 @@ void Trace::set_variable(Date &time, VariableType *type, Container *container, D
     delete_opt(opt);
 }
 
-void Trace::add_variable(Date &time, VariableType *type, Container *container, Double value, map<std::string, Value *> &opt) {
+void Trace::add_variable(Date &time, VariableType *type, Container *container, Double value, map<string, Value *> &opt) {
     if (container && type)
         container->add_variable(time, type, value);
     
@@ -240,7 +293,7 @@ void Trace::add_variable(Date &time, VariableType *type, Container *container, D
     delete_opt(opt);
 }
 
-void Trace::sub_variable(Date &time, VariableType *type, Container *container, Double value, map<std::string, Value *> &opt) {
+void Trace::sub_variable(Date &time, VariableType *type, Container *container, Double value, map<string, Value *> &opt) {
     if (container && type)
         container->sub_variable(time, type, value);
     
@@ -251,7 +304,7 @@ void Trace::sub_variable(Date &time, VariableType *type, Container *container, D
     delete_opt(opt);
 }
 
-void Trace::start_link(Date &time, LinkType *type, Container *ancestor, Container *source, EntityValue *value, String key, map<std::string, Value *> &opt) {
+void Trace::start_link(Date &time, LinkType *type, Container *ancestor, Container *source, EntityValue *value, String key, map<string, Value *> &opt) {
     if (ancestor && type && source)
         ancestor->start_link(time, type, source, value, key, opt);
     
@@ -259,7 +312,7 @@ void Trace::start_link(Date &time, LinkType *type, Container *ancestor, Containe
         _max_date = time;
 }
 
-void Trace::end_link(Date &time, LinkType *type, Container *ancestor, Container *destination, EntityValue */*value*/, String key, map<std::string, Value *> &opt) {
+void Trace::end_link(Date &time, LinkType *type, Container *ancestor, Container *destination, EntityValue */*value*/, String key, map<string, Value *> &opt) {
     if (ancestor && type && destination)
         ancestor->end_link(time, destination, key, opt);
     
@@ -268,31 +321,37 @@ void Trace::end_link(Date &time, LinkType *type, Container *ancestor, Container
 }
 
 void Trace::finish() {
-    std::stack<Container *> containers;
-    for (std::list<Container *>::const_iterator i = _root_containers.begin();
-        i != _root_containers.end();
-        i++)
+    stack<Container *> containers;
+    Container::VectorIt        i   = _root_containers.begin();
+    Container::VectorIt const &end = _root_containers.end();
+
+    for (; i != end; i++)
         containers.push(*i);
     
     while (!containers.empty()) {
         Container * c = containers.top();
         containers.pop();
         c->finish(_max_date);
-        for (std::list<Container *>::const_iterator i = c->get_children()->begin();
-            i != c->get_children()->end();
-            i++)
+
+	{
+	  Container::VectorIt        i   = c->get_children()->begin();
+	  Container::VectorIt const &end = c->get_children()->end();
+
+	  for (; i != end; i++)
             containers.push(*i);
+	}
     }
 }
 
-const list <Container *> *Trace::get_root_containers() const {
+const Container::Vector *Trace::get_root_containers() const {
     return &_root_containers;
 }
 
-void Trace::get_all_containers(std::list <Container *> &list_to_fill) const {
-    for (list<Container *>::const_iterator it = _root_containers.begin();
-         it != _root_containers.end();
-         it ++) {
+void Trace::get_all_containers(Container::Vector &list_to_fill) const {
+    Container::VectorIt        it  = _root_containers.begin();
+    Container::VectorIt const &end = _root_containers.end();
+
+    for (; it != end; it ++) {
         list_to_fill.push_back(*it);
         // Recursivity to add the children names
         add_containers(list_to_fill, *it);
@@ -300,12 +359,12 @@ void Trace::get_all_containers(std::list <Container *> &list_to_fill) const {
 
 }
 
-void Trace::add_containers(list <Container *> &containers, const Container *parent) const {
+void Trace::add_containers(Container::Vector &containers, const Container *parent) const {
+    const Container::Vector           *children  = parent->get_children();
+    Container::VectorIt        it  = children->begin();
+    Container::VectorIt const &end = children->end();
 
-    const list<Container *> *children = parent->get_children();
-    for (list<Container *>::const_iterator it = children->begin();
-         it != children->end();
-         it ++) {
+    for (; it != end; it ++) {
         // We create the node and we do the recursivity
         containers.push_back(*it);
         add_containers(containers , *it);
@@ -316,8 +375,12 @@ const list <StateType *> *Trace::get_state_types() const {
     return &_state_types;
 }
 
+const list <EventType *> *Trace::get_event_types() const {
+    return &_event_types;
+}
+
 template<class T>
-static T *search_tree(std::string name, T *el) { // Before : String name
+static T *search_tree(string name, T *el) { // Before : String name
     if (el->get_name() == name)
         return el;
     else {
@@ -334,6 +397,24 @@ static T *search_tree(std::string name, T *el) { // Before : String name
     return 0;
 }
 
+template<class T>
+static T *search_treeV(string name, T *el) { // Before : String name
+    if (el->get_name() == name)
+        return el;
+    else {
+        T *r;
+        typename vector<T *>::const_iterator it_end = el->get_children()->end();
+        for (typename vector<T *>::const_iterator it = el->get_children()->begin();
+             it != it_end;
+             ++ it) {
+            r = search_treeV<T>(name, *it);
+            if (r)
+                return r;
+        }
+    }
+    return 0;
+}
+
 ContainerType *Trace::search_container_type(String name) const {
     ContainerType *r;
     
@@ -360,13 +441,12 @@ Container *Trace::search_container(String name) const {
     if (std_name == "0")
         return 0;
     
-    const list<Container *>::const_iterator &it_end = _root_containers.end();
-    for (list<Container *>::const_iterator it = _root_containers.begin();
-        it != it_end;
-        ++ it) {
+    Container::VectorIt        it  = _root_containers.begin();
+    Container::VectorIt const &end = _root_containers.end();
+    for( ; it != end; ++ it) {
         r = search_tree<Container>(std_name, *it);
         if (r)
-            return r;
+             return r;
     }
      
     return 0;
@@ -466,7 +546,7 @@ EntityValue *Trace::search_entity_value(String name, EntityType *entity_type) co
     }
     Name alias_name;
     alias_name.set_name(name.to_string());
-    EntityValue *value = new EntityValue(alias_name, entity_type, map<std::string, Value*>());
+    EntityValue *value = new EntityValue(alias_name, entity_type, map<string, Value*>());
     entity_type->add_value(value);
     return value;
 }
@@ -475,7 +555,12 @@ Date Trace::get_max_date(){
   return _max_date;
 }
 
-void Trace::set_selected_container(std::vector<const Container *> * c) {
+int Trace::get_depth(){
+    return _depth;         
+}
+
+
+void Trace::set_selected_container(vector<const Container *> * c) {
     _selected_container = c;
 }
 
@@ -486,7 +571,7 @@ void Trace::set_interval_constrained(Interval * i) {
     _interval_constrained = i;
 }
 
-vector<const Container *>* Trace::get_selected_container() {
+vector<const Container *> *Trace::get_selected_container() {
     return _selected_container;
 }
 
@@ -502,20 +587,76 @@ double Trace::get_filter() {
     return _filter;
 }
 
-void Trace::get_all_variables(std::map<std::string, Variable *> &map_to_fill) {
-    list <Container *> all_containers;
+void Trace::get_all_variables(map<string, Variable *> &map_to_fill) {
+    Container::Vector all_containers;
     get_all_containers(all_containers);
-    list<Container *>::const_iterator all_container_end = all_containers.end();
-    
-    for (list<Container *>::const_iterator cont_it = all_containers.begin() ;
-         cont_it != all_container_end ; cont_it ++) {
-        const std::map<VariableType *, Variable *> *variables = (*cont_it)->get_variables();
+
+    Container::VectorIt const &end = all_containers.end();
+    Container::VectorIt        it  = all_containers.begin();
+    for ( ; it != end ; it ++) {
+        const map<VariableType *, Variable *> *variables = (*it)->get_variables();
         // We store each variable
-        for(std::map<VariableType *, Variable *>::const_iterator var_it = variables->begin() ;
+        for(map<VariableType *, Variable *>::const_iterator var_it = variables->begin() ;
             var_it != variables->end() ;
             var_it ++) {
-            string name = (*cont_it)->get_name().to_string() + " " + (*var_it).first->get_name().to_string();
+            string name = (*it)->get_name().to_string() + " " + (*var_it).first->get_name().to_string();
             map_to_fill[name] = (*var_it).second;
         }
     }
 }
+
+void Trace::get_states_colors(map<string, Color * > &nc_map, Container *parent) {
+    Container::Vector containers;
+    // We look for each container
+    if(parent)
+        add_containers(containers, parent);
+    else // We add all
+        get_all_containers(containers);
+
+    for(Container::VectorIt it = containers.begin();
+        it != containers.end(); ++ it) {
+        BinaryTree<StateChange> *states;
+        states = (*it)->get_states();
+        if(states && states->get_root()) {
+            get_state_names_colors_list(nc_map, states->get_root()); 
+        }
+    }
+}
+/* Fill name_map with the names of the state contained in sub-tree node */
+void get_state_names_colors_list(map<string, Color *> &name_map, Node<StateChange> *node) {
+    if (!node)
+        return;
+    
+    /* handle the node */
+    const State *left_st = node->get_element()->get_left_state();
+    if (left_st) {
+        const EntityValue *value = left_st->get_value();
+        const string name = value->get_name().to_string();
+        Color *color = NULL;
+        if(value->get_extra_fields()->find(string("Color")) != value->get_extra_fields()->end()) {
+            color = (Color *)value->get_extra_fields()->find(string("Color"))->second;
+        }
+
+        name_map[name] = color;
+    }
+    
+    const State *right_st = node->get_element()->get_right_state();
+    if (right_st) {
+        const EntityValue *value = right_st->get_value();
+        const string name = value->get_name().to_string();
+
+        Color *color = NULL;
+        if(value->get_extra_fields()->find(string("Color")) != value->get_extra_fields()->end()) {
+            color = (Color *)value->get_extra_fields()->find(string("Color"))->second;
+        }
+
+        name_map[name] = color;
+    }
+    
+    /* handle its left child */
+    get_state_names_colors_list(name_map, node->get_left_child());
+
+    /* handle its right child */
+    get_state_names_colors_list(name_map, node->get_right_child());
+}
+
